<?php

/********************************************
 *** General wiki utility functions       ***
 ********************************************/ 

/**
 * Switches the current blog to the wiki blog (if present)
 */
function switch_to_wiki_blog( $group_id ){
	$meta_key = 'wiki_blog_id';
	$blog_id = groups_get_groupmeta($group_id, $meta_key);
	switch_to_blog($blog_id);
}

/**
 * Sets up wp to act in loop mode for a single post i.e. avoiding the need
 * to set up a loop to loop through many posts when only one needs to be 
 * handled.
 *
 * @param array Post to be set up as the postdata
 */ 
function start_in_the_loop($loopPost) {
	global $post, $wp_query;
	$wp_query->in_the_loop = true;
	$post = $loopPost;
	setup_postdata($post);
}
/**
 * Ends a wp in loop mode.
 */
function end_in_the_loop() {
	global $wp_query;
	$wp_query->in_the_loop = false;
}

/**
 * Creates and echos a horizontal list of links of pages in a wiki blog.
 *
 * @param array Highlight the given post with the style 'selected-page'.
 */
function wiki_page_links( $groupID , $highlightPost = null ) {

	global $bp;
	
	query_posts('post_type=page&orderby=menu_order&order=ASC');
	
	// This will be set to true if any of the pages pass the user_can_view_wiki_page check
	$anyPagesVisible = false;
	
	$output .= "<div id='wiki-nav'>";
	$output .= "<ul id='wiki-pages'>";
	while (have_posts()) : the_post();
		$currentPost = get_post(get_the_ID());
		if (user_can_view_wiki_page( get_the_ID() , $groupID ) ) {
			$isSelected = '';
			if( ( $currentPost->post_name == $highlightPost->post_name ) && ( $bp->action_variables[0] != '' ) ) {
				$isSelected = ' class="selected-page"';
			}
			
			$output .= "<li".$isSelected."><a href='".bpgw_get_permalink( $groupID , $currentPost->ID )."'>";
			$output .= "".$currentPost->post_title;
			$output .= "</a></li>";
			// Set $anyPagesVisible to true so that we know to return a non-false value
			$anyPagesVisible = true;
		}
	endwhile;
	
	$output .= "</ul>";
	$output .= "<div class='clean'></div>";
	$output .= "</div>";
	
	// If user can see any of these pages, return the html for the page links, else return false
	if ( $anyPagesVisible ) {
		return $output;
	} else {
		return false;
	}
}
/**
 * Inspects the URL and attempts to identify and return the currently selected
 * page using the $bp->action_variable array.
 *
 * @returns array The selected page or null if none found.
 */
function get_selected_page_from_url() {
	global $bp;
	// This will be returned
	$selectedPage = null;
	
	// Grab all pages for the wiki-blog that exist sorted by menu_order
	$allPages = get_pages(array('sort_column' => 'menu_order'));

	// Get the last part of the slug, which may be the selected page_name
	$selectedPageName = $bp->action_variables[0];

	// Check if the action_variable has returned a valid page_name
	foreach($allPages as $page) {
		if($page->post_name  == $selectedPageName)
		{
			$selectedPage = $page;
		}
	}
	
	// If no matching page found return nothing.  
	// User should see home page instead
	
	return $selectedPage;
}


function to_wiki_date_format($timeStamp) {
	$formatedTimeStamp = sprintf('%1$s at %2$s',
				 format_date($timeStamp,'d F, Y'), 
				 format_date($timeStamp,'H:i'));
	return $formatedTimeStamp;
}

function format_date($timeStamp, $format) {
	$date = mysql2date($format, $timeStamp);
	return apply_filters('wiki_format_date', $date, $format);
}


/***************************************************
 *** Action privilege functions                  ***
 ***************************************************/ 
 
/**
 * Checks current user against current group and $postID wiki meta settings
 * @Param int $postID id of the post
 * @Returns true if able to, false if not
 * Updated to not require a switch to blog
 */
 function user_can_edit_wiki_page( $postID, $groupID ) {
	if(empty($postID) || empty($groupID)) 
		return false;
	
	global $bp;
	
	$blogID = groups_get_groupmeta( $groupID , 'wiki_blog_id' );
	
	// If the page is set to disabled, no edit or viewing is allowed
	if ( bpgw_get_blog_post_metadata( $blogID , $postID , 'wiki_page_enabled' ) == 0 ) {
		return false;
	}
	
	$postEditSettings = bpgw_get_blog_post_metadata( $blogID , $postID , 'wiki_edit_access' );
	
	// If user is banned, return false straight away.  Otherwise, do the full user access checks
	if (groups_is_user_banned( $bp->loggedin_user->id , $groupID )) return false;

	switch ( $postEditSettings ) {
		case 'all-members':
			if 
			(
				groups_is_user_admin( $bp->loggedin_user->id , $groupID )	||
				groups_is_user_mod( $bp->loggedin_user->id , $groupID )		||
				groups_is_user_member( $bp->loggedin_user->id , $groupID ) 	
			) 
			return true;
			break;
		case 'moderator-only':
			if 
			(
				groups_is_user_admin( $bp->loggedin_user->id , $groupID )	||
				groups_is_user_mod( $bp->loggedin_user->id , $groupID ) 	
			) 
			return true;
			break;
		case 'admin-only':
			if 
			(
				groups_is_user_admin( $bp->loggedin_user->id , $groupID )	
			) 
			return true;
			break;
		default:
			return false;
	}
}

function user_can_view_wiki_page( $postID, $groupID ) {

	global $bp;
	
	$blogID = groups_get_groupmeta( $groupID , 'wiki_blog_id' );

	// If the page is set to disabled, no edit or viewing is allowed
	if ( bpgw_get_blog_post_metadata( $blogID , $postID , 'wiki_page_enabled' ) == 0 ) {
		return false;
	}
	
	$postViewSettings = bpgw_get_blog_post_metadata( $blogID , $postID , 'wiki_view_access' , 1 );
	
	// If user is banned, return false straight away.  Otherwise, do the full user access checks
	if ( groups_is_user_banned( $bp->loggedin_user->id , $groupID ) ) return false;

	switch ( $postViewSettings ) {
		case 'public':
			return true;
			break;
		case 'member-only':
			if 
			(
				groups_is_user_admin( $bp->loggedin_user->id , $groupID )	||
				groups_is_user_mod( $bp->loggedin_user->id , $groupID )		||
				groups_is_user_member( $bp->loggedin_user->id , $groupID ) 	
			) 
			return true;
			break;
		default:
			return false;
	}
}

//placeholder
function wiki_page_comments_enabled( $wikiPost ) {

	if ( $wikiPost->comment_status == 'open') {
		return true;
	} else {
		return false;
	}
}

/********************************************
 *** Wiki display functions               ***
 ********************************************/ 

 
/**
 * Generates the contents box for a displayed wiki page from header <h#> tags.
 * Intended to be used with a filter on the the_contents tag.
 * 
 * Styling taken from wordpress_wiki plugin by Dan Milward, Thomas Howard, Allen Han.
 * See: http://wordpress.org/extend/plugins/wordpress-wiki/
 *
 * Contents list building based on code from:
 * http://www.wait-till-i.com/2010/01/06/the-table-of-contents-script-my-old-nemesis/
 *
 * @param string The content of the wiki page
 * @returns string The contents with a contents table pre-pended.
 */


function table_of_contents($content = '') {
	// If nothing to do, return nothing
	if(empty($content)) return $content;
	
	// Prepend our content with the place holder for the toc
	$content = '<div id="toc"></div>'.$content;

	// Grab all header tags
	preg_match_all("/<h([1-6])[^>]*>.*<\/h.>/Us",$content,$headlines);
	
	// If no headlines to index, don't bother displaying a contents table.
	if(empty($headlines[0])) return $content;
	
	// The class must match the javascript show/hide parameter
	$out = '<ul class="contents-index">'; 
	
	foreach($headlines[0] as $k=>$h){
	// DEBUGING
	//echo("LEVEL: " . $headlines[1][$k] . "<br />\n");

	 // Insert IDs into header tags of content to act as anchors
	 // But check for any pre-exisitng id's inside the header, try to match VERY carefully
	 if(strstr($h,' id=')===false){
	   $x = preg_replace('/>/',' id="head'.$k.'">',$h,1);
	   $content = str_replace_once($h,$x,$content);
	   $h = $x;
	 };
	 
	 // Strip out any tags nested inside the header so we are left with just text 
	 // (this gets rid of undesirable extras such as images which wreck the contents table)
	 $h = strip_tags($h, '<h1><h2><h3><h4><h5><h6>');
	 
	 // Create the link for the table of contents
	 $link = preg_replace('/<(\/)?h\d/','<$1a',$h);
	 $link = str_replace('id="','href="#',$link);

	 
	 /*** Handling of nested lists when generating the table of contents ***/
	 
	 // Add additional nesting for each higher level if the FIRST H tag isn't H1.
	 if($k==0 && $headlines[1][$k] > 1){
		 for($i = $headlines[1][$k]; $i>1; $i--) {
			$out.='<ul>'."\n";
		 }
	 }
	 
	 // Add additional nesting where we jump more than one level where we aren't at first tag
	 if($k>0 && ($headlines[1][$k] - $headlines[1][$k-1]) > 1) {
		 for($i = $headlines[1][$k]; $i>($headlines[1][$k-1]+1); $i--) {
			$out.='<ul>'."\n";
		 }
	 }
	 
	 // If the previous H is less than the current H start an new list
	 if($k>0 && $headlines[1][$k-1]<$headlines[1][$k]){
	   $out.="\n".'<ul>'."\n";
	 }
	 
	 // Always start a new entry with a new list item
	 $out .= '<li>'.$link.'';
	 
	 // If next H exists and next H is less than current H, close list item
	 if($headlines[1][$k+1] && $headlines[1][$k+1]<$headlines[1][$k]){
	   $out.='</li>'."\n";
	   // Then close all nested lists until we reach the level of the next H 
	   // (e.g. If going from H4 to H2 it will loop twice)
	   $currentLevel = $headlines[1][$k];
	   $nextLevel = $headlines[1][$k+1];
	   for($i = $currentLevel; $i>$nextLevel; $i--) {
		  $out.='</ul>'."\n".'</li>'."\n";
	   }
	 }
	 
	 // If next H exists and next H is same level as current H, close current list item
	 if($headlines[1][$k+1] && $headlines[1][$k+1] == $headlines[1][$k]){
	   $out.='</li>'."\n";
	 }
	}
	// Always end by closing the first list item
	$out.='</li>'."\n".'</ul>'."\n";
	
	// Wrap the whole contents table in a tags for presentation
	$out = '<div class="toc"><h3>Contents</h3>'.
	'<p> &#91; <a class="show" onclick="toggle_hide_show(this, \'contents-index\')">hide</a> &#93; </p>'.
	"\n".$out.'</div>'."\n";
	
	return str_replace('<div id="toc"></div>',$out,$content);
}

function str_replace_once($needle, $replace, $haystack) { 
   // Looks for the first occurence of $needle in $haystack 
   // and replaces it with $replace. 
   $pos = strpos($haystack, $needle); 
   if ($pos === false) { 
       // Nothing found 
       return $haystack; 
   } 
   return substr_replace($haystack, $replace, $pos, strlen($needle)); 
} 
	
/********************************************
 *** Wiki revisions functions             ***
 ********************************************/ 
	
/**
 * Gets a number of revisions for the current post or returns all 
 * if $maxRevisions is set to less than zero.  For this to work correctly,
 * this must be called from 'within the loop'.
 *
 * @param int $maxRevisions The number of revisions to return or -1 to return all
 *
 * @return array The revision posts.
 */
function get_revisions($maxRevisions = 5) {
	global $post;
	
	// Check for any abnomalities with current post and if so return empty array
	if ( !$post = get_post( $post->ID ) ){
		return array();
	}
	if($post->post_type == 'revision' && ($post->post_parent > 0)) {
		if(!$post = get_post( $post->post_parent )) {
			return array();
		}
	}
	
	// Setup the args
	$revisionArgs = array(
		'numberposts' => $maxRevisions, 
		'post_type' => 'revision',
		'order' => 'DESC', 
		'orderby' => 'date');
	
	if ( !$revisions = wp_get_post_revisions( $post->ID, $revisionArgs)) {
		return array();
	}
	return array_values($revisions);
}

/**
 * Returns an HTML title bar to display above the revisions list.
 *
 * @param array $args For specifying how to format the title bar
 *
 * @return Formated HTML containg the revisions title bar.
 */ 
function get_revisions_titlebar( $group_id , $args = array()) {
	global $post;
	$defaults = array(
		'title_text' => 'Post Revisions',
		'restore_button' => false,
		'history_button' => false,
		'restore_link' => '',
		'restore_link_attr' => '',
		'history_link' => bpgw_get_permalink( $group_id , $post->ID ).'revision/',
		'history_link_attr' => '',		
		'echo' => false
	);
	$args = wp_parse_args( $args, $defaults );
	
	$buttonHTML = '';
	if($args['history_button']) {
		$buttonHTML .= 	'<div class="generic-button group-button public">'.
							'<a href="'.$args['history_link'].'" '.$args['history_link_attr'].'>'.
							'<span class="history">Revision History</span></a></div>';
	}
	if($args['restore_button']) {
		$buttonHTML .= 	'<div class="generic-button group-button public">'.
							'<a href="'.$args['restore_link'].'" '.$args['restore_link_attr'].'>'.
							'<span class="restore">Restore Version</span></a></div>';
	}
	$output = "<div class='titlebar'><h4>{$args['title_text']}</h4>{$buttonHTML}</div>";
	
	if($args['echo'])
		echo $output;
		
	return $output;
}


/**
 * Returns a formated list of links to revisions of the current post.  
 * By default, only the first 5 revisions are displayed.
 * Intended to be used with a filter on the the_contents tag. For this to 
 * work correctly, this must be called from 'within the loop'.
 *
 * @param int $content $group_id The group ID the user is part of
 * @return string Content with the revisions list appended to the end.
 */
function wiki_page_revisions_summary( $group_id , $maxRevisions = 5){
	global $post;

	$revisions = get_revisions(-1);
	$revisionsTotalCount = count($revisions) + 1; // Add 1 for current revision
	if(empty($revisions)) {
		return $content;
	}
	
	$titleBarArgs = array(
		'title_text' => 'Post Revisions Summary <span>('.$revisionsTotalCount.')</span>',
		'history_button' => true,
		'history_link' => bpgw_get_permalink( $group_id , $post->ID ).'revision/', 
		'echo' => false
	);
	
	// Remove revisions beyond our maximum number
	$i = 0;
	foreach ($revisions as $key => $revision) {
		if($maxRevisions > 0 && $i >= $maxRevisions) {
			unset($revisions[$key]);
		}
		$i++;
	}
	$revisions = array_values($revisions);
	$revisionsSummaryCount = count($revisions) + 1; // Add 1 for current revision
	
	// Create the HTML fragment
	$output .= "<div class='wiki-revisions'>";
	$output .= get_revisions_titlebar( $group_id , $titleBarArgs);
	if($revisionsTotalCount > $revisionsSummaryCount) {
	$output .= "<p class='wiki-meta'>Only $revisionsSummaryCount of $revisionsTotalCount revisions shown. ".
			   "Select the 'Revision History' button to view a list of all revisions.";
	}
	$output .= format_revisions_summary_list( $group_id , $revisions);
	$output .= "</div>";

	return $content.$output;
}

/**
 * Formats a given array of revision posts into an HTML list of links.
 *
 * @param array $revisions The revision posts to format.
 * @return string A HTML unordered list containing formated links to the revisions.
 */
function format_revisions_summary_list( $group_id , $revisions ) {
	if(empty($revisions)) {
		return '';
	}
	
	global $post;
	$rows = '';

	foreach ($revisions as $revision) {
		$title = format_revision_as_title($revision);
		$rows .= "\t<li>$title</li>\n";
	}	
	$link = bpgw_get_permalink( $group_id , $post->ID );
	
	$output = "<ul class='post-revisions'>\n";
	$output .= "\t<li>".format_current_revision_as_title($group_id)." - <a href='".$link."edit/'>Edit this page</a></li>\n";
	$output .= $rows;
	$output .= "</ul>";
	return $output;
}

/**
 * Generates a formatted HTML table containing more detailed information about all revisions of a wiki page.
 *
 * @param int $groupID The group ID of the user
 * @param array $currentPage The parent wiki page to generate information for
 * @param array $revisions An array of all revisions of the current page.
 *
 * @return string HTML output ready for display.
 */
function format_revisions_as_comparison_table($groupID, $currentPage, $revisions) {
	if(empty($revisions)) {
		return '';
	}
	
	// Get details of the current revision
	$revisionRows = get_revision_table_row($groupID, $currentPage, $revisions, $currentPage);
	// Get details of all other revisions
	foreach ($revisions as $revision) {	
		$revisionRows .= get_revision_table_row($groupID, $currentPage, $revisions, $revision);
	}	
	
	// Build the table
	$output .= '<table class="revisions-compare">'.$newLine;
	$output .= '<thead>'.$newLine;
	$output .= '<tr>'.$newLine;
	$output .= '<th colspan="2" class="col-radio"></th>'.$newLine;
	$output .= '<th class="col-revision">Revision Date</th>'.$newLine;
	$output .= '<th class="col-author">Author</th>'.$newLine;
	$output .= '<th class="col-notes">Notes</th>'.$newLine;
	$output .= '</tr>'.$newLine;
	$output .= '</thead>'.$newLine;
	$output .= '<tbody>'.$newLine;
	$output .= $revisionRows;
	$output .= '</tbody>'.$newLine;
	$output .= '</table>'.$newLine;
	
	return $output;
}
/**
 * Generates a HTML table row containing information about a specific wiki page revision.
 *
 * @see format_revisions_as_comparison_table();
 * @param int $groupID The group ID of the user
 * @param array $currentPage The parent wiki page to generate information for
 * @param array $revisions An array of all revisions of the current page.
 * @param array $targetRevision The revision to generate information about
 *
 * @return string HTML table row output.
 */
function get_revision_table_row($groupID, $currentPage, $revisions, $targetRevision) {
	// Declared for convenience
	$newLine = "\n";
	$pageID = $currentPage->ID;
	$revisionID = $targetRevision->ID;
	
	// Create data for current page
	$postReference = wiki_revision_date_reference($targetRevision, true);
	$postAuthor = bp_core_get_userlink($targetRevision->post_author);
	$versionNum = get_version_number($currentPage, $revisions, $revisionID);
	$notes = "Version ".$versionNum;
	
	if($pageID == $revisionID)
		$notes .= " [ Current revision ]";
	else if($versionNum == 1) 
		$notes .= " [ Initial version ]";
	
	// Get any restore history to add to the notes
	$restoredFromId = get_restored_revision_source_id($pageID, $revisionID);
	if(isset($restoredFromId)) {
		$baseLink = bpgw_get_permalink($groupID, $pageID);
		$restoredFromLink = $baseLink."revision/$restoredFromId#item-nav";
		$restoredFromVersionNum = get_version_number($currentPage, $revisions, $restoredFromId);
		$notes .= " - Restored from <a href='$restoredFromLink'>version $restoredFromVersionNum</a>";
	} 
	
	// Create the table row
	$altStyle = '';
	// Need to ensure alt style is added based on count from first table row
	if((count($revisions)-($versionNum-1)) %2 == 0) {
		$altStyle = ' class="alt"';
	}
	$newLine = "\n";
	$output = "<tr$altStyle>".$newLine;
	$output .= '<td class="col-radio"><input type="radio" name="left" value="'.$revisionID.'" /></td>'.$newLine;
	$output .= '<td class="col-radio"><input type="radio" name="right" value="'.$revisionID.'" /></td>'.$newLine;
	$output .= '<td class="col-revision">'.$postReference.'</td>'.$newLine;
	$output .= '<td class="col-author">'.$postAuthor.'</td>'.$newLine;
	$output .= '<td class="col-notes">'.$notes.'</td>'.$newLine;
	$output .= '</tr>'.$newLine;
	
	return $output;
}



/**
 * Gets the sequential version number of a revision from a set of revisions.  
 * This number is based on the order the revisions in the given array.  Lower positions
 * in the revisions array are assumed to be older and therefore revision[0] is version 1.
 * The current revision is assumed to be one more than the number of revisions.
 *
 * @param array $currentPage The parent wiki page to generate information for
 * @param array $revisions An array of all revisions of the current page.
 * @param array $targetRevisionID The ID of the revision to get the version number for
 *
 * @return int A whole number representing the sequential position of the target revision.
 */
$revisionToVersionMap = null;
function get_version_number($currentPage, $revisions, $targetRevisionID = null) {
	if($revisionToVersionMap == null) {
		$versionNum = count($revisions);
		$revisionToVersionMap = array();
		// Add in current version
		$revisionToVersionMap["".$currentPage->ID] = $versionNum+1;
		// Add in all revision versions
		foreach ($revisions as $revision) {
			$revisionToVersionMap["".$revision->ID] = $versionNum;
			$versionNum--;
		}
	}
	if($targetRevisionID == null)
		return $revisionToVersionMap[$currentPage->ID];
	else
		return $revisionToVersionMap[$targetRevisionID];
}

/**
 * Looks for any restored version information for a given wiki page revision to check 
 * if a given revision is a restored copy of another revision.
 *
 * @param int $pageID The parent wiki page to check the restore history for
 * @param int $revisionID The ID of the revision to check if it is a restored 
 * copy of another revision.
 *
 * @return int The ID of the revision the given revision was restored from 
 * or null if nothings was found.
 */
$restorePointsMap = null;
function get_restored_revision_source_id($pageID, $revisionID) {
	if($restorePointsMap == null) {
		$restoreMeta = get_post_meta($pageID, WIKI_M_PAGE_RESTORED, false);
		$restorePointsMap = array();
		foreach($restoreMeta as $key=>$value) {
			$restorePair = explode('-', $value, 2);
			$restorePointsMap["".$restorePair[0]] = $restorePair[1];
		}
	}
	return $restorePointsMap[$revisionID];
}


/**
 * Validates whether a page being used in a wiki comparison is valid.
 * Valid pages are (1) The current page revision itself, or (2) any
 * revision of the current page i.e. its parent is the current page.
 *
 * @param array $currentPage The current page of the wiki.
 * @param array $revisionForCompare The candidate revision in the compare.
 * @return boolean True if valid, false otherwise.
 */
function validateRevisionForCompare($currentPage,$revisionForCompare) {
	if( $revisionForCompare == null || empty($revisionForCompare)) {
		// No post retrieved
		return false;
	}
	if($currentPage->ID == $revisionForCompare->ID) {
		// The revision IS the current page so must be OK
		return true;
	}
	if( $revisionForCompare->post_type == 'revision' && 
		$revisionForCompare->post_parent == $currentPage->ID) {
			// Revision belongs to current page
			return true;
	}
	// Assume all other cases are false
	return false;
}	

/** Currently not used.  Code use under review */
function clean_revisions_diff_output($diff_text) {
	
	/*** a new dom object ***/
    $dom = new domDocument;
    /*** load the html into the object ***/
    $dom->loadHTML($diff_text);
    /*** discard white space ***/
    $dom->preserveWhiteSpace = false;
    /*** the table by its tag name ***/
    $tables = $dom->getElementsByTagName('table');
    /*** get all rows from the table ***/
    $rows = $tables->item(0)->getElementsByTagName('tr');

	$output = '';
	
    /*** loop over the table rows ***/
    foreach ($rows as $row) {
        /*** get each column by tag name ***/
        $cols = $row->getElementsByTagName('td');
		//print_r($cols);
        /*** echo the values ***/
		 foreach ($cols as $col) {
			echo  strip_tags( html_entity_decode($col->nodeValue) , '<del><tr><td>' );
			echo "<br />";
		}
    } 
	
	
	$diff_text = strip_tags( html_entity_decode($diff_text) , '<del><tr><td>' );
	 htmlentities($diff_text);
	return $diff_text;
}



/**
 * Formats a given revision post into an HTML title which can be optionally 
 * hyperlinked to the revision text.
 *
 * @param array $revision The revision post to format into a title.
 * @param boolean $hyperlink Whether or not to make the title 
 * hyperlinked to the revision text.
 *
 * @return string A HTML title for the revision.
 */
function format_revision_as_title($revision, $hyperlink = true) {
	$titlef = _c( '%1$s by %2$s|post revision 1:datetime, 2:name' );
	$date = wiki_revision_reference($revision, $hyperlink);
	if($hyperlink) {
		$name = bp_core_get_userlink($revision->post_author);
	}
	else {
		$userdata = get_userdata($revision->post_author);
		$name = $userdata->display_name;
	}
	
	return sprintf( $titlef, $date, $name );
}
/**
 * Formats the current post into an HTML title which can be optionally 
 * hyperlinked to the revision text.  For this to work correctly,
 * this must be called from 'within the loop'.
 *
 * @param boolean $hyperlink Whether or not to make the title 
 * hyperlinked to the revision text.
 *
 * @return string A HTML title for the revision.
 */ 
function format_current_revision_as_title($group_id, $hyperlink = true) {
	global $post;
	$link = bpgw_get_permalink($group_id, $post->ID);
	$name = bp_core_get_userlink($post->post_author);
		
	$output = 'Current revision';
	if($hyperlink) {
		$output = "<a href='".$link."'>".$output."</a>";
	}
	$output .= " by ".$name;

	return $output;
}


/**
 * Formats the title/link of a wiki revision page reference as a version date.
 *
 * Taken from wordpress_wiki plugin by Dan Milward, Thomas Howard, Allen Han.
 * See: http://wordpress.org/extend/plugins/wordpress-wiki/
 *
 * @param array $revision The revision post the generate a title/link for.
 * @param boolean $hyperlink Whether or not to format text as a hyperlink to 
 * display the wiki revision.  Defaults to true.
 *
 * @return A formated HTML link to the revision post.
 */
function wiki_revision_reference($revision, $hyperlink = true) {
	if (!$revision = get_post($revision))
		return $revision;

	if (!in_array($revision->post_type, array( 'post', 'page', 'revision' )))
		return false;

	$autosavef = __( '%s [Autosave]' );
	$currentf	= __( '%s [Current Revision]' );
	$date = wiki_revision_date_reference($revision, $hyperlink);

	if ( !wp_is_post_revision( $revision ) )
		$date = sprintf( $currentf, $date );
	elseif ( wp_is_post_autosave( $revision ) )
		$date = sprintf( $autosavef, $date );

	return $date;
}


function wiki_revision_date_reference($revision, $hyperlink = true) {
	global $bp;
	$group_id = $bp->groups->current_group->id;
	
	$date = to_wiki_date_format($revision->post_modified_gmt . ' +0000' );
	if($revision->post_parent == 0){
		$post_id = $revision->ID;
	} else {
		$post_id = $revision->post_parent;
	}
	$link = bpgw_get_permalink($group_id, $post_id)."revision/".$revision->ID;
	if($hyperlink)
		$date = "<a href='$link#item-nav'>$date</a>";
	return $date;
}


function carry_over_restore_postmeta($postID) {
	// Check to see if the current version of the page has a restore flag
	$restoredFromID = get_restored_revision_source_id($postID, $postID);

	// Do we need to copy across any restore point history into the meta data
	if(!empty($restoredFromID)) {
	
		// Immediately grab the next lastest revision which should be what previously was the current version
		$revisions = wp_get_post_revisions($postID, array('numberposts' => 1, 'post_type' => 'revision',
														  'order' => 'DESC', 'orderby' => 'date'));
		if(!empty($revisions)) {
			$latestRevisionID = $revisions[key($revisions)]->ID;
			// Delete the old metadata arecord for the current post
			delete_post_meta($postID, WIKI_M_PAGE_RESTORED, $postID.'-'.$restoredFromID); 
			// Update the post meta restore history to assign restored ID to the new revision
			add_post_meta($postID, WIKI_M_PAGE_RESTORED, $latestRevisionID.'-'.$restoredFromID); 
		}
	}	
}

/********************************************
 *** Wiki comments functions              ***
 ********************************************/ 
 

 /**
 * Gets the comments attached to the current post.  For this to work correctly,
 * this must be called from 'within the loop'.
 *
 * @param string $status Only return comments with this status.
 * 'hold' - unapproved comments
 * 'approve' - approved comments
 * 'spam' - spam comments
 * [empty value] all comments.
 * 
 * @return An array of any comments.
 */
 function get_wiki_comments( $wikiPage, $status = '' ) {

	if($wikiPage->post_type == 'revision' && ($post->post_parent > 0)) {
		if(!$wikiPage = get_post( $wikiPage->post_parent )) {
			return array();
		}
	}
	$args = array(
    'status' => $status,
    'orderby' => 'comment_date_gmt',
    'order' => 'DESC',
    'post_id' => $wikiPage->ID);
	if ( !$comments = get_comments($args)) {
		return array();
	}
	return $comments;
}



 /**
 * Returns an HTML title bar to display above the comments list.
 *
 * @param int The number of comments being displayed.
 * @param boolean Whether or not to show a count of the number of comments.
 *
 * @return Formated HTML containg the revisions title bar.
 */ 
function get_comments_titlebar($wikiPageID, $commentsCount = '', $titleText = 'Comments') {
	if(is_numeric($commentsCount)) {
		$commentsCount = 	'<span>('.$commentsCount.')</span>';
	}
	return "<div class='titlebar'><h4>{$titleText} {$commentsCount}</h4></div>";
}
 
 
 /**
 * Formats a given array of comments posts into a pretty HTML list.
 *
 * @param array $comments The comments  to format.
 *
 * @return string A HTML unordered list containing formated comments.
 */
function format_comments_as_list($comments, $itemsPerPage, $group_id) {
	if(empty($comments)) {
		return '';
	}
	
	$totalPages = wiki_get_total_pages_count($comments, $itemsPerPage);
	$currentPage = wiki_get_comments_page_from_url($totalPages);
	$comments = wiki_get_page_of_items($comments, $currentPage, $itemsPerPage);
	
	$rows = '';
	$i = 0;
	foreach ($comments as $comment) {
		$oddeven = ($i%2) ? "odd" : "even";
		$id = $comment->comment_ID ;
		$userID = $comment->user_id;
		$formatedComment = bpgw_format_comment($comment, $group_id);
		$rows .= "\t<li id='comment-{$id}' class='comment byuser comment-author-{$userID} {$oddeven} alt'>{$formatedComment}</li>\n";
		$i++;
	}	
	
	$output = "<div id='comments'><ul class='commentlist'>\n";
	$output .= $rows;
	$output .= "</ul></div>";
	return $output;
}

/**
 * Formats a comment with HTML markup for display.
 * 
 * @param $comment The comment to format.
 *
 * @return string HTML formated content.
 */
function bpgw_format_comment($comment, $group_id) {
	global $bp;
	$commentID = $comment->comment_ID;
	$authorID = $comment->user_id;
	$commentTimeStamp = $comment->comment_date;
	
	$userData = get_userdata($authorID);
	$userDisplayName = $userData->display_name;
	$userProfileLink = bp_core_get_userlink($authorID);
	
	$postTimeStamp = to_wiki_date_format($commentTimeStamp);
	$commentAdminOptions = bpgw_get_comment_moderator_options($comment, $group_id);
	
	$output = "<div id='div-comment-$commentID' class='comment-body'>";
	$output .= "<div class='columns-two'>\n<div class='left'>\n";
	$output .= "<div class='comment-author vcard'>";
	$output .= get_avatar($authorID, 16, $default, $userDisplayName);
	$output .= "<cite class='fn'>$userProfileLink</cite> <span class='says'>says:</span></div>";
	$output .= "</div>\n";
	$output .= "<div class='right'>\n";
	$output .= "<div class='comment-date'>$postTimeStamp</div>";
	$output .= $commentAdminOptions;
	$output .= "</div>\n</div>\n";
	$output .= convert_smilies($comment->comment_content);
	$output .= "</div>";

	return $output;
}

function bpgw_get_comment_moderator_options($comment, $group_id) {
	global $bp;
	
	$comment_id = $comment->comment_ID;
	$options_html = '';
	// Allow admins, mods or current user to delete own comments
	if(!bpgw_can_moderate_comment($comment, $group_id)) {
		return $options_html;
	}
	$deleted_comment_list = bpgw_get_deleted_comment_ids($group_id);
	if(in_array($comment_id, $deleted_comment_list)) {
		return $options_html;
	}
	$delete_confirm = 'return confirm(\''.
					 'Really delete this comment?\n\n'.
					 'To proceed with deleting this comment, please select OK to confirm.\')';
	
	$options_html .= '<div class="comment-options">';
	$options_html .= '<form id="comment-mod-form" class="standard-form" method="post" action="'. WIKI_S_COMMENT_OPTIONS .'">';
	if ( function_exists('wp_nonce_field') )
		$options_html .= wp_nonce_field('wiki_comment_options');
	$options_html .= '<input type="hidden" id="group_id" name="group_id" value="'.$group_id.'" />';
	$options_html .= '<input type="hidden" id="post_id" name="post_id" value="'.$comment->comment_post_ID.'" />';
	$options_html .= '<input type="hidden" id="comment_id" name="comment_id" value="'.$comment_id.'"/>';
	$options_html .= '<input id="comment-delete" class="comment-delete button" '.
						 'type="submit" value="Delete" name="submit" onclick="'.$delete_confirm.'" />';
	$options_html .= '</form>';
	$options_html .= '</div>';
	
	return $options_html;
}
/**
 * Checks to see if user is allowed to delete a comment.  Allows admins
 * and moderators to delete comments or current user to delete own comments.
 *
 * @param array $comment The comment to check
 * @param int $group_id The group ID the logged in user belongs to
 * @return boolean True if the user is allowed to delete the comment, false otherwise.
 */
function bpgw_can_moderate_comment($comment, $group_id) {
	global $bp;
	if(!is_user_logged_in()) 
		return false;
	else
		return ( groups_is_user_admin( $bp->loggedin_user->id , $group_id )	||
				 groups_is_user_mod( $bp->loggedin_user->id , $group_id ) ||
				 $bp->loggedin_user->id == $comment->user_id );
} 

function wiki_get_comments_page_from_url($totalPages) {
	global $bp;
	$wikiActionVar = $bp->action_variables[1];
	$commentsPagedPrefix = "comment-page-";
	
	if(string_begins_with($wikiActionVar, $commentsPagedPrefix)) {
		$pageNumber = substr($wikiActionVar, strlen($commentsPagedPrefix));

		// Sanity check the parsed value
		if(!is_numeric($pageNumber) || $pageNumber < 0) return 1; // Default to first page
		if($pageNumber > $totalPages) return $totalPages; // Default to last page
		
		return $pageNumber;
	}
	// Nothing viable found so default to first page
	return 1;
}



function bpgw_add_comment_meta($blog_id, $comment_id, $meta_key, $meta_value, $unique = false) {
	if ( !$meta_key )
		return false;

	global $wpdb;

	// expected_slashed ($meta_key)
	$meta_key = stripslashes($meta_key);

	if ( $unique && $wpdb->get_var( $wpdb->prepare( "SELECT meta_key ".
													"FROM {$wpdb->base_prefix}%d_commentmeta ".
													"WHERE meta_key = %s AND ".
													"comment_id = %d", 
													$blog_id, $meta_key, $comment_id ) ) ) {
		return false;
	}

	$meta_value = maybe_serialize( stripslashes_deep($meta_value) );

	$sql = $wpdb->prepare( "INSERT INTO {$wpdb->base_prefix}%d_commentmeta ".
						   "( comment_id, meta_key, meta_value ) ".
						   "VALUES ( %d, %s, %s )", 
						   $blog_id, $comment_id, $meta_key, $meta_value );
	$wpdb->query($sql);
	wp_cache_delete($comment_id, 'comment_meta');

	return true;
}

$_deleted_comment_ids = null; // Should really use wp cache
function bpgw_get_deleted_comment_ids($group_id) {
	if($_deleted_comment_ids == null) {
		global $wpdb;
		$blog_id = groups_get_groupmeta( $group_id , 'wiki_blog_id' );
		$rows = $wpdb->get_results( $wpdb->prepare("SELECT comment_id ".
												"FROM {$wpdb->base_prefix}%d_commentmeta ".
												"WHERE meta_key = %s ",
												$blog_id, WIKI_M_COMMENT_DELETED_CONTENT ), ARRAY_A ); 
		$_deleted_comment_ids = array();
		if(empty($rows)) {
			return $_deleted_comment_ids;
		}
		// Otherwise, populated with what we pulled out the database
		$i = 0;
		foreach($rows as $k=>$id) {
			$_deleted_comment_ids[$i] = $id['comment_id'];
			$i++;
		}
	}
	return $_deleted_comment_ids;
}


function string_begins_with($string, $search) {
    return (strncmp($string, $search, strlen($search)) == 0);
}


function wiki_get_page_of_items($array_of_items, $page, $items_per_page) {
		$total_items = count($array_of_items);
		$last_item = $page*$items_per_page;
		$first_item = $last_item-$items_per_page;
		
		$page_of_items = array();
		for($i = 0; $i < $items_per_page; $i++) {
			if(isset($array_of_items[$first_item + $i])) {
				$page_of_items[$i] = $array_of_items[$first_item + $i];
			}
		}
		return $page_of_items;
}


/**
 * Calculate the total number of pages need to display items in an array 
 * (pages, comments etc), based on a given per page quota.
 * Based on WordPress get_comment_pages_count()
 *
 * @param array $items Optional array of objects. 
 * @param int $per_page Optional items per page. Defaults to 10
 * @return int Number of pages.
 */
function wiki_get_total_pages_count($items = null, $items_per_page) {
	 
	if (empty($items) )
		return 0;

	$count = ceil( count( $items ) / $items_per_page );
	return $count;
}

function wiki_paginate_links($baseURI, $items, $items_per_page, $args = array()) {
	$totalPageCount = wiki_get_total_pages_count($items, $items_per_page);
	$defaults = array(
		'base' => $baseURI,
		'format' => '',
		'total' => $totalPageCount, // total number of pages to output
		'current' => wiki_get_comments_page_from_url($totalPageCount), 
		'echo' => true,
		'add_fragment' => '#comments'
	);
	$defaults['base'] = user_trailingslashit(trailingslashit($baseURI) . 'comment-page-%#%', 'commentpaged');

	$args = wp_parse_args( $args, $defaults );
	$page_links = paginate_links( $args );
		
	if ( $args['echo'] )
		echo $page_links;
	else
		return $page_links;
	
}



/********************************************
 *** Activity Stream Functions              ***
 ********************************************/ 
 


 
// word-safe and tag-safe function for reducing length of a string
function substrws( $text, $len=180 ) {

if( (strlen($text) > $len) ) {

	$whitespaceposition = strpos($text," ",$len)-1;

	if( $whitespaceposition > 0 )
		$text = substr($text, 0, ($whitespaceposition+1));

		// close unclosed html tags
		if( preg_match_all("|<([a-zA-Z]+)>|",$text,$aBuffer) ) {

			if( !empty($aBuffer[1]) ) {

				preg_match_all("|</([a-zA-Z]+)>|",$text,$aBuffer2);

				if( count($aBuffer[1]) != count($aBuffer2[1]) ) {

					foreach( $aBuffer[1] as $index => $tag ) {

						if( empty($aBuffer2[1][$index]) || $aBuffer2[1][$index] != $tag)
							$text .= '</'.$tag.'>';
					}
				}
			}
		}
	}

	return $text;
} 


function wiki_groups_record_activity( $args = '' ) {
	global $bp;

	if ( !function_exists( 'bp_activity_add' ) )
		return false;

	$defaults = array(
		'user_id' => $bp->loggedin_user->id,
		'content' => false,
		'action' => false,
		'primary_link' => false,
		'component' => 'groups',
		'component_name' => $bp->groups->id,
		'component_action' => false,
		'type' => false,
		'item_id' => false,
		'secondary_item_id' => false,
		'recorded_time' => gmdate( "Y-m-d H:i:s" ),
		'hide_sitewide' => $hide_sitewide
	);

	$r = wp_parse_args( $args, $defaults );
	extract( $r, EXTR_SKIP );

	return bp_activity_add( array( 'user_id' => $user_id, 'content' => $content, 'action' => $action, 'primary_link' => $primary_link, 'component' => 'groups', 'component_name' => $component_name, 'type' => $type, 'component_action' => $component_action, 'item_id' => $item_id, 'secondary_item_id' => $secondary_item_id, 'recorded_time' => $recorded_time, 'hide_sitewide' => $hide_sitewide ) );
}


// Returns a 'permalink' to a wiki page based on the group slug + page slug.  
// Requires group_id, page_id
function bpgw_get_permalink( $group_id , $post_id ) {
	global $bp;
	
	$wiki_group = new BP_Groups_Group( $group_id , false , false );	 	// Create group object so we can use bp_get_group_permalink
	$wiki_url = bp_get_group_permalink ( $wiki_group ); 				// Get the link to the group
	$blog_id = groups_get_groupmeta( $group_id, 'wiki_blog_id' ); 		// Get the group wiki blog ID
	$wiki_post = get_blog_post ( $blog_id, $post_id ); 					// Retrieve the post
	$wiki_post_url = $wiki_url . 'wiki/' . $wiki_post->post_name . '/';	// Append the post_name to $url and add a trailing slash
	
	return $wiki_post_url; 												// Return wiki post permalink in the format http://siteurl/groupname/wiki/postname
}




// Returns a 'permalink' to a wiki page based on the group slug + page slug.  
// Requires group_id, page_id
function bpgw_get_group_id_for_current_blog() {
	global $wpdb, $blog_id, $bp;

	$meta_key = 'wiki_blog_id';
	
	$groupMeta = $wpdb->get_row( $wpdb->prepare( "SELECT group_id FROM " . $bp->groups->table_name_groupmeta . " WHERE meta_key = %s AND meta_value = %d", $meta_key , $blog_id ) , ARRAY_A ); 

	return $groupMeta['group_id'];				// Return the groupID number (non-array string)
}





// Kills the /wp-admin page for wiki blogs.  Might be useful to re-enable at some point but for now it can just cause problems
// Also *temporarily* grants the current_user the right to upload files if this is a blog post that they have edit rights on
function bpgw_wp_admin_overrides() {
	global $current_user, $bp, $pagenow;
	
	if ( get_option( 'this_is_a_wiki_blog' , 0 ) == 1 ) { 
	
		$groupID = bpgw_get_group_id_for_current_blog();
		$userCanEditThisPage = user_can_edit_wiki_page( $_REQUEST['post_id'] , $groupID ); // post_id included in the media file request.  If not set, will return false
		
		if ( $userCanEditThisPage && ( $pagenow == 'media-upload.php' || $pagenow == 'async-upload.php' ) ) {	
				$current_user->allcaps['upload_files'] = true; // Enable upload rights for the remainder of this script process
			} else {
				die ( "No access" ); // Other admin areas are disabled or user isn't allowed to edit this page.  Get the fudge out
			}
	}
}
add_action('admin_init', 'bpgw_wp_admin_overrides');






//-------------------------------------------------------------------------------------------
// Get metadata for a specific blog/post without using 'the enemy' - switch_to_blog
// Based on get_blog_post (wpmu function)
// NOTE: Assumes that this is a unique meta key!
// @returns -> object, "bpgw_get_blog_post_metadata($args)->meta_value" to get the meta value that was set
function bpgw_get_blog_post_metadata( $blog_id, $post_id, $meta_key ) {
	global $wpdb;

	$key = $blog_id."-".$post_id."-".$meta_key."-blog_post_meta"; // The key to store/retrieve this value from the wp_cache (saves on server load)
	$metaValue = wp_cache_get( $key, "bpgw-post-meta" ); // Checks to see if this was value was already cached on the server
	if( $metaValue == false ) { // If there's nothing for it in the cache do a db lookup
		$metaValue = $wpdb->get_row( $wpdb->prepare("SELECT meta_value FROM {$wpdb->base_prefix}%d_postmeta WHERE post_id = %d && meta_key= %s", $blog_id, $post_id,  $meta_key ) ); // Don't forget to sanitise that naughty potential sql injection data
		wp_cache_add( $key, $metaValue, "bpgw-post-meta", 120 ); // Expires from cache in 120 seconds
	}

	return $metaValue->meta_value;
}
//-------------------------------------------------------------------------------------------











//-------------------------------------------------------------------------------------------
// The filter means that when "query_vars" is processed by wordpress our function 
// ...is called and it parses the post/get vars and adds them to the global $public_query_vars
// @returns -> the $public_query_vars so they can be used by the rest of the site.  Don't forget this or you'll kill the site :)
function bpgw_form_vars($public_query_vars) {

	$public_query_vars[] = 'bpgw_group_id';
	$public_query_vars[] = 'bpgw_post_id';
	$public_query_vars[] = 'wiki_edit_lock';
	$public_query_vars[] = 'wiki_frontend_page_create';
	$public_query_vars[] = 'bpgw_newpage_title';

	return ($public_query_vars);
}
add_filter('query_vars', 'bpgw_form_vars');
//-------------------------------------------------------------------------------------------











//-------------------------------------------------------------------------------------------
// Call this at the start of a processing function to get the values of the variables so we can do stuff with them
// Not sure if the file data one will break/be dodgy.  Need to test+investigate... "intestigate"
// @returns -> array of the vars.  These are also stored in the $bpgw_form_vars global
function bpgw_get_form_vars(){  
    global $bpgw_form_vars;  
	
    if(get_query_var('bpgw_group_id')) {  
        $bpgw_form_vars['group_id'] = mysql_real_escape_string(get_query_var('bpgw_group_id'));  
    }
	
    if(get_query_var('bpgw_post_id')) {  
        $bpgw_form_vars['post_id'] = mysql_real_escape_string(get_query_var('bpgw_post_id'));  
    }
	
    if(get_query_var('wiki_edit_lock')) {  
        $bpgw_form_vars['wiki_edit_lock'] = mysql_real_escape_string(get_query_var('wiki_edit_lock'));  
    }
	
    if(get_query_var('wiki_frontend_page_create')) {  
        $bpgw_form_vars['wiki_frontend_page_create'] = mysql_real_escape_string(get_query_var('wiki_frontend_page_create'));  
    }
	
    if(get_query_var('bpgw_newpage_title')) {  
        $bpgw_form_vars['bpgw_newpage_title'] = mysql_real_escape_string(get_query_var('bpgw_newpage_title'));  
    }
	
    return $bpgw_form_vars;  
} 
//-------------------------------------------------------------------------------------------






//-------------------------------------------------------------------------------------------
// This function is called by an ajax POST every 10 seconds whilst a user is editing a wiki page
// Updates the post meta with an entry showing the time of edit and user id/name ( with a timeout of 15 seconds )
// This will mean that after 15 seconds of not viewing the page, the person will not appear in the 'also editing this document' list
// TODO: Add a warning if someone else saves their page whilst you work on this
// TODO: Remove the meta data for a user 'currently editing' once they save (and/or on unload).  Would make feedback more instantaneous + cleaner
function bpgw_editing_now() {
    global $bpgw_form_vars, $bp, $wpdb;
	bpgw_get_form_vars();  
	if ( $bpgw_form_vars['group_id'] && $bpgw_form_vars['post_id'] && $bpgw_form_vars['wiki_edit_lock'] ) {
		/* IMPORTANT - WE NEED TO STICK A NONCE VERIFY HERE!!!! */
		$group_id = $bpgw_form_vars['group_id'];
		$post_id  = $bpgw_form_vars['post_id'];
		$blog_id  = groups_get_groupmeta( $group_id , 'wiki_blog_id' );
		
		if ( user_can_edit_wiki_page( $post_id, $group_id ) ) {
			$rowAlreadyExists = 0;
			$output = '';
			// Add this person to the cache/list of people editing the wikipage, echo the list
			$postEditData = $wpdb->get_results( $wpdb->prepare("SELECT meta_id, meta_value FROM {$wpdb->base_prefix}%d_postmeta WHERE post_id = %d && meta_key = 'bpgw_edit_lock'", $blog_id, $post_id ) , ARRAY_A );
			if ( $postEditData ) {
				$firstOutput=true;
				foreach ( $postEditData as $postEdit ) {
		//explode the string with &
		//$editData['meta_value'][0]==userID, $editData['meta_value'][1]==timestamp, 
		//$editData['meta_value'][2]==Authorname, $editData['meta_value'][3]==warningflag (usually not set)
		//if the time is within 15 seconds of now add them to the $output string		
		// when save, set other people's warningflag to zero
		// do check when user id is current user id - if warningflag is set and timestamp <30s ago, this post has been saved
		// also do a check - if timestamp >120, delete this row
					$editData = explode( "&" , $postEdit['meta_value'] );
					if ( $editData[1] != $bp->loggedin_user->id ) { // If this wasn't the current user ...
						if ( time() - $editData[2] < 15 ) { // ...AND time less than 15 seconds ago, add them to the list of people editing
							if ( $firstOutput ) {
								$output = '<div class="editing-now">Also editing this page: </div>';
								$firstOutput = false;
							}
							$output .= '<div class="author-editing-now">' . $editData[3] . '</div>';
						}
					} else {
						// If there's an entry for the current user we'll need to update the entry rather than create a new one
						$rowAlreadyExists = $postEdit['meta_id']; 
						// Check to see if the warning flag is set for this user and if their current timestamp is within last 15 seconds
						if ( ( time() - $editData[2] < 15 ) && ( $editData[4] == 1 ) ) { 
							$output .= '<div id="page-save-warning"></div>';
						}						
					}
				}
				if(!empty($output)) {
					$output = '<div id="editing-now-warnings">'.$output.
							  '<div class="edit-now-advice"><strong>Warning:</strong> If another user saves after you, they may overwrite your changes.</div></div>';
				}
				echo $output; // echo out the author names
			}
		// Now we need to update the meta data to reflect the fact that the user is editing this wiki page
		if ( $rowAlreadyExists ) { // There's already an entry for this user so we need to update it
			$meta_value = '&' . $bp->loggedin_user->id . '&' . time() . '&' . $bp->loggedin_user->fullname;
			$wpdb->query( $wpdb->prepare( "	UPDATE {$wpdb->base_prefix}%d_postmeta
											SET meta_value = '%s'
											WHERE meta_id = %d", 
												$blog_id, $meta_value, $rowAlreadyExists ) );
		
		} else { // There is no previous entry for the user so we can create a new one
			$meta_value = '&' . $bp->loggedin_user->id . '&' . time() . '&' . $bp->loggedin_user->fullname;
			$wpdb->query( $wpdb->prepare( "	INSERT INTO {$wpdb->base_prefix}%d_postmeta
											( post_id, meta_key, meta_value )
											VALUES ( %d, 'bpgw_edit_lock', %s )", 
												$blog_id, $post_id, $meta_value ) );
		}
		die; // kill the process so the rest of WP doesn't load
		}
	} // If we get to this point the page request isn't our submitted form.  The rest of WP will load normally now
}
add_action('template_redirect', 'bpgw_editing_now');  
//-------------------------------------------------------------------------------------------





//-------------------------------------------------------------------------------------------
// This function is called when a user submits a page creation request from the wiki-index area (only if this is enabled in wiki admin)
// Creates the page in the wiki/blog.  Returns el zilcho for now, as we haven't got the time to add any fancy feedback
// TODO: Feedback, redirect user to newly created page
function bpgw_frontend_page_create() {
    global $bpgw_form_vars, $bp, $wpdb;
	bpgw_get_form_vars();  
	if ( $bpgw_form_vars['group_id'] && $bpgw_form_vars['bpgw_newpage_title'] && $bpgw_form_vars['wiki_frontend_page_create'] ) {
		/* IMPORTANT - WE NEED TO STICK A NONCE VERIFY HERE!!!! */
		$group_id = $bpgw_form_vars['group_id'];
		$blog_id  = groups_get_groupmeta( $group_id , 'wiki_blog_id' );
		if ( groups_get_groupmeta( $group_id , 'wiki_member_page_create' ) == 1 && groups_is_user_member( $bp->loggedin_user->id , $group_id ) ) {
				switch_to_blog($blog_id);
				// Create post object
				$wikiPost = array(
					'post_content' => '', //The full text of the post.
					'post_excerpt' => '', //For all your post excerpt needs.
					'post_status' => 'publish', //Set the status of the new post
					'post_title' => 'Post Title', //The title of the post
					'post_type' => 'page', //Wiki only uses pages, not posts
					'menu_order' => '0' //Order page will appear in nav menu - lower number appears first
				); 
				$wikiPost['post_title'] = mysql_real_escape_string($bpgw_form_vars['bpgw_newpage_title']);
				$wikiPost['menu_order'] = 9999; // work around to make the page appear at the end of the page listing
				// insert the wiki page
				$wikiPostID = wp_insert_post( $wikiPost );
				// Set the meta data for the page to control view/edit access
				add_post_meta( $wikiPostID , 'wiki_view_access' , 'public' , 1 );
				add_post_meta( $wikiPostID , 'wiki_edit_access' , 'all-members' , 1 );
				add_post_meta( $wikiPostID , 'wiki_page_enabled' , 1 , 1 );
				echo 'done';
		}
		die; // kill the process so the rest of WP doesn't load
	} // If we get to this point the page request isn't our submitted form.  The rest of WP will load normally now
}
add_action('template_redirect', 'bpgw_frontend_page_create');  
//-------------------------------------------------------------------------------------------
















//-------------------------------------------------------------------------------------------
// This function is called as a filter for post revision content.  It simply adds an extra ../ to the start of relative links
// ...this navigates back up from the /groups/ directory that buddypress uses for groups
// @returns $content
function bpgw_revision_image_link_fix( $content ) {
	$content=str_ireplace('../../../../','../../../../../',$content);
	return $content;
}
//-------------------------------------------------------------------------------------------




// Slugify a title
function slugified_title ($text) {
		// replace non letter or digits by -
		$text = preg_replace('~[^\\pL\d]+~u', '-', $text);
		// trim
		$text = trim($text, '-');
		// transliterate
		if (function_exists('iconv')) {
			$text = iconv('utf-8', 'us-ascii//TRANSLIT', $text);
		}
		// lowercase
		$text = strtolower($text);
		// remove unwanted characters
		$text = preg_replace('~[^-\w]+~', '', $text);
		if (empty($text)) {
			return 'n-a';
		}
		return $text;
	}








function bpgw_create_wiki( $groupWikiDataArray , $wikiViewAccess , $wikiEditAccess , $wikiMemberPageCreateAllowed ) {
	global $bp, $wpdb, $current_site;
	
	// Title only really seen in the site admin backend.  Not used in the buddypress frontend at all.
	$wikiTitle = 'GroupWiki_'.$bp->groups->current_group->id;
	$wikiMeta = '';
	$domain = $current_site->domain;
	$wikiPath = $current_site->path . $bp->groups->current_group->slug . '-wiki/';

	// Create the blog to be used for wiki
	$wikiID = wpmu_create_blog( $domain, $wikiPath, $wikiTitle, $bp->loggedin_user->id, $wikiMeta, $wpdb->siteid );

	if ( $wikiID ) {
		// Add groupmeta to attach blog id to group id
		groups_update_groupmeta( $bp->groups->current_group->id , 'wiki_blog_id' , $wikiID );
		// Add groupmeta to set wiki blog thing to 'enabled'
		groups_update_groupmeta( $bp->groups->current_group->id , 'wiki_enabled' , $wikiID );
		// Add groupmeta to set allow standard members to create pages
		groups_update_groupmeta( $bp->groups->new_group_id , 'wiki_member_page_create' , $wikiMemberPageCreateAllowed );
		
		// Create post object
		$wikiPost = array(
			'post_content' => '', //The full text of the post.
			'post_excerpt' => '', //For all your post excerpt needs.
			'post_name' => '', // The URI/URL/slug of the post (based on title + slugified)
			'post_status' => 'publish', //Set the status of the new post
			'post_title' => '', //The title of the post
			'post_type' => 'page', //Wiki only uses pages, not posts
			'menu_order' => '0' //Order page will appear in nav menu - lower number appears first
		); 

		// Select the blog/wiki and insert pages/posts for each wiki page
		switch_to_blog($wikiID);

		// but first, delete all the default created pages
		$allPages = get_all_page_ids();
		if ($allPages) {
			foreach( $allPages as $pageID ) {
				wp_delete_post( $pageID , true );
			}
		}

		// and delete all the default created posts
		$allPosts = get_posts( 'orderby=title' );
		if ($allPosts) {
			foreach( $allPosts as $onePost ) {
				wp_delete_post( $onePost->ID , true );
			}
		}

		update_option( 'this_is_a_wiki_blog' , 1 ); 
		update_option( 'template' , 'wikioverride' ); 
		update_option( 'stylesheet' , 'wikioverride' ); 

		$wikiPageCount = 1;

		foreach ($groupWikiDataArray as $groupWikiData) {
			
			// Modify page data for this post
			$wikiPost['post_name'] = slugified_title($groupWikiData);
			if ( $wikiPost['post_name'] == '' ) continue; // Skip the page creation if the user didn't fill in a title
			$wikiPost['post_title'] = esc_attr($groupWikiData);
			$wikiPost['menu_order'] = $wikiPageCount;
			$wikiPageCount++;
			// insert the wiki page
			$wikiPostID = wp_insert_post( $wikiPost );
			// Set the meta data for the page to control view/edit access
			add_post_meta( $wikiPostID , 'wiki_view_access' , $wikiViewAccess , 1 );
			add_post_meta( $wikiPostID , 'wiki_edit_access' , $wikiEditAccess , 1 );
			add_post_meta( $wikiPostID , 'wiki_page_enabled' , 1 , 1 );
		}

		// Record wiki creation in the activity stream
		$group = new BP_Groups_Group( $bp->groups->current_group->id , false , false );
		groups_record_activity( array(
			'content' => '<a href="' . bp_get_group_permalink( $group ) . '">' . attribute_escape( $group->name ) . '</a> activated their <a href="'.bp_get_group_permalink( $group ).'wiki">group wiki</a>:<span class="time-since">%s</span></p>',
			'primary_link' => bp_get_group_permalink( $group ),
			'component_action' => 'new_wiki_created',
			'item_id' => $group->id
		) );	

		groups_update_groupmeta( $bp->groups->current_group->id, 'last_activity', time() );

		// Remove any usermeta for the blog/wiki - otherwise it will appear in various blog menus
		$meta_key_caps  = $wpdb->base_prefix . $wikiID . '_capabilities';
		$meta_key_level = $wpdb->base_prefix . $wikiID . '_user_level';
		delete_usermeta( $bp->loggedin_user->id, $meta_key_caps );
		delete_usermeta( $bp->loggedin_user->id, $meta_key_level );		
		$wpdb->query( $wpdb->prepare( "DELETE FROM {$bp->blogs->table_name} WHERE user_id = %d AND blog_id = %d", $bp->loggedin_user->id , $wikiID ) );	

		// Take it back mahhn..take it back to the original blog.
		restore_current_blog();
	}
	
	return $wikiID;
}




?>