<?php

include(WP_PLUGIN_DIR.'/buddypress-group-wiki/bp-groupwiki.inc.php');

class Wiki extends BP_Group_Extension {	

	function wiki() {
		
		global $bp;
		
		$this->name = 'Wiki';
		$this->slug = 'wiki'; 
		
		$this->create_step_position = 16;
		$this->nav_item_position = 41;
		
		$this->visibility = 'public';
		
		// Only enable the group nav item if the groupwiki has been activated
		if ( groups_get_groupmeta( $bp->groups->current_group->id , 'wiki_enabled' ) > 0 ) {
			$this->enable_nav_item = true;
		} else {
			$this->enable_nav_item = false;
		}
      
		add_action ( 'wp_print_styles' , array( &$this , 'add_wiki_stylesheet' ) );
		add_action ( 'wp_print_scripts' , array( &$this , 'add_wiki_script' ) );
		add_action ( 'wp_print_scripts' , array( &$this , 'add_wiki_tinymce' ) );

		// Make it so that private groups also display the wiki nav item, even if people aren't members
		// Buddypress has checks in place to stop the same nav item being shown twice, so no need to worry about that situation
		add_action( 'wp', array( &$this , 'bpgw_setup_wikinav' ), 2 );
		add_action( 'admin_menu', array( &$this , 'bpgw_setup_wikinav' ), 2 );

	}
	
	// This only adds the item in the case of a group being private.  This will possibly mean the item will be added twice, but BP automagically deals with that for us
	public function bpgw_setup_wikinav()  {

		global $bp, $current_user;

		if ( $bp->current_component == $bp->groups->slug && $bp->is_single_item ) {
			
			if ( $bp->groups->current_group->status == 'private' && groups_get_groupmeta( $bp->groups->current_group->id , 'wiki_enabled' ) > 0 )  {
			
				$group_link = $bp->root_domain . '/' . $bp->groups->slug . '/' . $bp->groups->current_group->slug . '/';
				
				if ( groups_get_groupmeta( $bp->groups->current_group->id , 'wiki_enabled' ) > 0 ) {
					bp_core_new_subnav_item( array( 'name' => ( !$this->nav_item_name ) ? $this->name : $this->nav_item_name, 'slug' => $this->slug, 'parent_slug' => BP_GROUPS_SLUG, 'parent_url' => bp_get_group_permalink( $bp->groups->current_group ) . '/', 'position' => $this->nav_item_position, 'item_css_id' => 'nav-' . $this->slug, 'screen_function' => array( &$this, '_display_hook' ), 'user_has_access' => $this->enable_nav_item ) );
				}
			}
		}
	}
		
	public function add_wiki_stylesheet() {
        $myStyleUrl = WP_PLUGIN_URL . '/buddypress-group-wiki/css/style.css';
        $myStyleFile = WP_PLUGIN_DIR . '/buddypress-group-wiki/css/style.css';
        if (file_exists($myStyleFile)) {
            wp_register_style('groupwiki-style', $myStyleUrl);
            wp_enqueue_style('groupwiki-style');
        }
    }
	
	public function add_wiki_script() {
        $myScriptUrl = WP_PLUGIN_URL . '/buddypress-group-wiki/js/script.js';
        $myScriptFile = WP_PLUGIN_DIR . '/buddypress-group-wiki/js/script.js';
        if (file_exists($myScriptFile)) {
            wp_register_script('groupwiki-script', $myScriptUrl);
            wp_enqueue_script('groupwiki-script');
        }
    }
	
	public function add_wiki_tinymce() {
        $myScriptUrl = WP_PLUGIN_URL . '/buddypress-group-wiki/js/tiny_mce/tiny_mce.js';
        $myScriptFile = WP_PLUGIN_DIR . '/buddypress-group-wiki/js/tiny_mce/tiny_mce.js';
        if (file_exists($myScriptFile)) {
            wp_register_script('groupwiki-tinymce', $myScriptUrl);
            wp_enqueue_script('groupwiki-tinymce');
        }
    }	

	
	function create_screen() {
		if ( !bp_is_group_creation_step( $this->slug ) )
			return false;
		
		require('content-pages/wiki-create.php');
		
		
		wp_nonce_field( 'groups_create_save_' . $this->slug );
	}

	function create_screen_save() {
		global $bp, $wpdb, $domain, $current_site;
		
		check_admin_referer( 'groups_create_save_' . $this->slug );
		
		// Wiki enable checkbox ticked?
		$wikiEnableCheckboxTicked = $_POST['groupwiki-enable-wiki'];
		
		// Wiki allow standard members to create pages ticked?
		// If yes, we'll be setting the groupmeta value to one
		if ( $_POST['groupwiki-member-page-create'] ) {
			$wikiMemberPageCreateAllowed = 1;
		} else {
			$wikiMemberPageCreateAllowed = 0;
		}
		
		if ( !groups_get_groupmeta( $bp->groups->new_group_id, 'wiki_blog_id' )  && $wikiEnableCheckboxTicked ) {

			// process the page titles
			$groupWikiDataArray = array_unique($_POST[ "wikiPageTitle" ]);
			// process the privacy choices ready to be inserted as post meta data
			$wikiViewAccess = $_POST['wiki-privacy'];
			// process the privacy choices ready to be inserted as post meta data
			$wikiEditAccess = $_POST['wiki-edit-rights'];

			bpgw_create_wiki( $groupWikiDataArray , $wikiViewAccess , $wikiEditAccess , $wikiMemberPageCreateAllowed);

		}
	}

	function edit_screen() {
		global $bp;
		if ( !bp_is_group_admin_screen( $this->slug ) )
			return false; 
			
		if ( !( groups_is_user_admin( $bp->loggedin_user->id , $bp->groups->current_group->id ) || is_site_admin() ) )
			return false;
			
		//call the admin screen file
		require('content-pages/wiki-admin.php');
		
		wp_nonce_field( 'groups_edit_save_' . $this->slug );
		
	}

	function edit_screen_save() {
		global $bp, $wpdb, $domain, $current_site;

		if ( !isset( $_POST['save'] ) )
			return false;
		
		check_admin_referer( 'groups_edit_save_' . $this->slug );
		
		$wikiEnableCheckboxTicked = $_POST['groupwiki-enable-wiki'];
		
		// Wiki allow standard members to create pages ticked?
		// If yes, we'll be setting the groupmeta value to one
		if ( $_POST['groupwiki-member-page-create'] ) {
			$wikiMemberPageCreateAllowed = 1;
		} else {
			$wikiMemberPageCreateAllowed = 0;
		}
		
		if ( groups_get_groupmeta( $bp->groups->current_group->id , 'wiki_blog_id' ) ) {
				
		//if wiki previously enabled:
			//process the wiki page settings
			//process any new page creation
			//set wiki enabled to 1 or 0		
			// Switch to the blog associated with this group
			
			$blogID = groups_get_groupmeta( $bp->groups->current_group->id , 'wiki_blog_id' );
			switch_to_blog($blogID);		
			
			// Enabled/disable the GroupWiki, depending on ye olde checkbox
			if (!isset($_POST['groupwiki-enable-wiki'])) {
				groups_update_groupmeta( $bp->groups->current_group->id , 'wiki_enabled' , 0 );
			} else {
				groups_update_groupmeta( $bp->groups->current_group->id , 'wiki_enabled' , $blogID );
			}
		
			// Add groupmeta to set allow standard members to create pages
			groups_update_groupmeta( $bp->groups->current_group->id , 'wiki_member_page_create' , $wikiMemberPageCreateAllowed );
				
			// Update the page title and comment settings
			$pageOptionsArray = $_POST['wikiPage'];
			foreach ( $pageOptionsArray as $pageOptions ) {
			
				$post = array();
				$post['ID'] = $pageOptions['id'];
				if ( is_numeric($pageOptions['order']) ) {
					$post['menu_order']  = $pageOptions['order'];
				}
				if  ( $pageOptions['title'] != '' ) { // If title isn't set then keep the old title.
					$post['post_title']  = esc_attr($pageOptions['title']);
				}
				if (!isset($pageOptions['comments'])) { // comments checkbox not checked, so close comments
					$post['comment_status'] = 'closed';
				} else { // comments checkbox IS checked, so open comments
					$post['comment_status'] = 'open';
				}
				
				// Get the old post 
				$oldPost = get_post( $post['ID'] );
				// Check to see if the title has been changed and is not null
				If ( $oldPost->post_title != $post['post_title'] ) {
					$titleChange = TRUE;
				} else {
					$titleChange = FALSE;
				}
				// Check to see if the comments setting has changed
				If ( $oldPost->comment_status != $post['comment_status'] ) {
					$commentChange = TRUE;
				} else {
					$commentChange = FALSE;
				}
				
				// If needed, temporarily turn off post revisions (turned on && title/comments same as previous)
				if ( WP_POST_REVISIONS == 1 && !$titleChange && !$commentChange  ) {
					remove_action('pre_post_update', 'wp_save_post_revision');
					wp_update_post( $post );
					add_action('pre_post_update', 'wp_save_post_revision');
				} else { // Title or comments have changed, or revisions not enabled anyhow.  
					wp_update_post( $post );
				}
				
				// Set the post meta to control view/edit access
				update_post_meta( $post['ID'] , 'wiki_view_access' , $pageOptions['privacy'] );
				update_post_meta( $post['ID'] , 'wiki_edit_access' , $pageOptions['edit'] );
				update_post_meta( $post['ID'] , 'wiki_page_enabled' , $pageOptions['enabled'] );
				
			}

			//Create post object
			$wikiPost = array(
				'post_content' => '', //The full text of the post.
				'post_excerpt' => '', //For all your post excerpt needs.
				'post_name' => '', // The URI/URL/slug of the post
				'post_status' => 'publish', //Set the status of the new post
				'post_title' => '', //The title of the post
				'post_type' => 'page', //Wiki only uses pages, not posts
				'menu_order' => '0' //Order page will appear in nav menu - lower number appears first
			); 
			
			// Create new pages if this section of form was filled in			
			if ( isset( $_POST['wikiPageTitle'] ) ) {
				
				// Set the default wiki view and edit rights
				$wikiViewAccess = 'public';
				$wikiEditAccess = 'all-members';			
				
				$groupWikiDataArray = $_POST['wikiPageTitle'];
				$wikiPageCount = count( $pageOptionsArray ) + 1; // count number of already created pages

				foreach ($groupWikiDataArray as $groupWikiData) {
					
					// Modify page data for this post
					$wikiPost['post_name'] = slugified_title($groupWikiData);
					if ( $wikiPost['post_name'] == '' ) continue; // Skip the page creation if the user didn't fill in a title
					$wikiPost['post_title'] = esc_attr($groupWikiData);
					$wikiPost['menu_order'] = $wikiPageCount;
					$wikiPageCount++;
					// insert the wiki page
					$wikiPostID = wp_insert_post( $wikiPost );
					// Set the meta data for the page to control view/edit access
					add_post_meta( $wikiPostID , 'wiki_view_access' , $wikiViewAccess , 1);
					add_post_meta( $wikiPostID , 'wiki_edit_access' , $wikiEditAccess , 1);
					add_post_meta( $wikiPostID , 'wiki_page_enabled' , 1 , 1 );
				}
				groups_update_groupmeta( $bp->groups->current_group->id, 'last_activity', time() );
			}		
			
			$wpdb->query( $wpdb->prepare( "DELETE FROM {$bp->blogs->table_name} WHERE user_id = %d AND blog_id = %d", 1 , $wikiID ) );	
			
			restore_current_blog();
			
			$success = 1;
			
		} elseif  ( $wikiEnableCheckboxTicked ) {	
			//if wiki ***NOT*** previously enabled:
					//create a blog for the wiki
					//do the prep work on that blog
					//process any new page creation
					//process the wiki page settings
					//set wiki enabled to 1
			
			if ( !groups_get_groupmeta( $bp->groups->current_group->id, 'wiki_blog_id' )  && $wikiEnableCheckboxTicked ) {

				// process the page titles
				$groupWikiDataArray = array_unique($_POST[ "wikiPageTitle" ]);
				// process the privacy choices ready to be inserted as post meta data
				$wikiViewAccess = $_POST['wiki-privacy'];
				// process the privacy choices ready to be inserted as post meta data
				$wikiEditAccess = $_POST['wiki-edit-rights'];

				$success = bpgw_create_wiki( $groupWikiDataArray , $wikiViewAccess , $wikiEditAccess );
			}

		} else {
			$success = TRUE;
			$noChanges = TRUE;
		}

		/* To post an error/success/no change message to the screen, use the following */
		if ( !$success && !$noChanges ) {
			bp_core_add_message( __( 'There was an error saving, please try again', 'buddypress' ), 'error' );
		} elseif ( $noChanges ) {
			bp_core_add_message( __( 'No changes were made to wiki settings', 'buddypress' ), 'error' );
		} else {
			bp_core_add_message( __( 'Settings saved successfully', 'buddypress' ) );
		}
		
		bp_core_redirect( bp_get_group_permalink( $bp->groups->current_group ) . '/admin/' . $this->slug );
	}

	function display() {
	
		global $bp;

		// Call the display screen  file
		require('content-pages/wiki-index.php');
	
	}

	function widget_display() { 
		// blank
	}
}

bp_register_group_extension( 'Wiki' );

?>