<?php

if ( 'POST' != $_SERVER['REQUEST_METHOD'] ) {
	header('Allow: POST');
	header('HTTP/1.1 405 Method Not Allowed');
	header('Content-Type: text/plain');
	exit;
}

/** - START - Sets up the WordPress Environment. */
$path = substr(__FILE__,0,strpos(__FILE__,"wp-content"));
require($path . '/wp-load.php' );
nocache_headers();
wp();
require_once( ABSPATH . WPINC . '/template-loader.php' );
/** - END - Sets up the WordPress Environment. */


include(WP_PLUGIN_DIR.'/buddypress-group-wiki/bp-groupwiki.inc.php');

// Validate the nonce
if (!wp_verify_nonce($_REQUEST['_wpnonce'], 'wiki_unfiltered_html_comment')) {
	die('Security check');
}

global $bp;

$groupID = $_POST['group_id'];
$postID = $_POST['comment_post_id'];
$blogID = groups_get_groupmeta( $groupID , 'wiki_blog_id' );


// Get the currently selected page
$wikiPage = get_blog_post( $blogID , $postID );
// Get post permalink for later redirection
$returnLink = bpgw_get_permalink( $groupID , $postID ); 
// Switch to blog and set the loop_post for the selected page 
switch_to_blog($blogID);
start_in_the_loop($wikiPage);

// Booleans to record progress
$canAddComments = false;
$commentAdded = false;	

// Permissions checks
$wikiCanView = user_can_view_wiki_page( $wikiPage->ID , $groupID );
$wikiCanComment = wiki_page_comments_enabled( $wikiPage );

// TODO: Check user id vs group membership 

// Check if user may add comments
if ( is_user_logged_in() && $wikiCanView && $wikiCanComment && ($_POST['save'] != 'Save')) { 
	$canAddComments = true;
}


// If the user has passed above checks, run through the save process
if ($canAddComments) {
	
	// Update the page with the new comment
	$commentText = $_POST['comment_text'];
	
	global $current_user;
    get_currentuserinfo(); // Places result inside global '$current_user'

	$commentdata = array();
	$commentdata['comment_post_ID'] = $wikiPage->ID;
	$commentdata['comment_author'] = $current_user->user_login;
	$commentdata['comment_author_email'] = $current_user->user_email ;
	$commentdata['user_ID'] = $current_user->ID;
	$commentdata['comment_author_IP'] = $_SERVER['REMOTE_ADDR'];
	$commentdata['comment_content'] = $commentText;
	$commentdata['comment_agent'] = $_SERVER['HTTP_USER_AGENT'];
	$commentdata['comment_type'] = 'comment';

	// Calling wp_new_comment also sanitizes the comment
	$commentID = wp_new_comment($commentdata);
	
	// Create the return link for a successful commenting
	$returnLink .= "#comment-$commentID";
	
	
	
	// Generate the activity stream update message
	$pageName = $wikiPage->post_title;
	$commentExcerpt = substrws($commentText, 300) . '...';
	$activityContent= $commentExcerpt;	
	$name = '<a href="' . $bp->loggedin_user->domain . '">' . $bp->loggedin_user->fullname . '</a>';
	$group = new BP_Groups_Group( $groupID , false , false );
	
	// Create the activity update
	$activityUpdate = $name . ' commented on the <a href="' . bp_get_group_permalink( $group ) . '">' .
					  attribute_escape( $group->name ) . '</a> wiki page <a href="' . $returnLink .
					  '">'.$pageName.'</a>: <span class="time-since">%s</span>';
	
	// Issue update based on privacy settings
	$pagePrivacy = get_post_meta( $postID , 'wiki_view_access' , 1 );
	
	
	/*David: the comment activity stream item should simply be on the sitewide feed based 
	on the page being public or not (with the override of the group being hidden)
	the notification email should preferably be to all group members, 
	assuming they have view or edit access on the page*/

	if ( $pagePrivacy == 'public' && $bp->groups->current_group->status != 'hidden' ) {
		// Record wiki creation in the activity stream, viewable by everyone
		wiki_groups_record_activity( array(
			'action' => $activityUpdate,
			'content' => $activityContent,
			'primary_link' => $returnLink,
			'component' => 'groups',
			'component_action' => 'new_wiki_comment',
			'type' => 'new_wiki_comment',
			'item_id' => $group->id,
			'secondary_item_id' => $bp->loggedin_user->id
		) );
	} 
	else {
		// Record wiki creation in the activity stream, hide sitewide
		wiki_groups_record_activity( array(
			'action' => $activityUpdate,
			'content' => $activityContent,
			'primary_link' => $returnLink,
			'component' => 'groups',
			'component_action' => 'new_wiki_comment',
			'type' => 'new_wiki_comment',
			'item_id' => $group->id,
			'secondary_item_id' => $bp->loggedin_user->id,
			'hide_sitewide' => true
		) );	
	}
	
	// Success feedback
	bp_core_add_message( __( 'Your comment has been saved.', 'buddypress' ) );
	// Update the last_activity time for this group
	groups_update_groupmeta( $groupID, 'last_activity', time() );
}
else { 
	// No access feedback
	bp_core_add_message( __( 'You do not have the required access to do that.', 'buddypress' ), 'error' );
}


// End the loop and restore blog
end_in_the_loop();
restore_current_blog();

// Send the user back to the page view screen
bp_core_redirect( $returnLink );
?>