<?php

if ( 'POST' != $_SERVER['REQUEST_METHOD'] ) {
	header('Allow: POST');
	header('HTTP/1.1 405 Method Not Allowed');
	header('Content-Type: text/plain');
	exit;
}

/** - START - Sets up the WordPress Environment. */
$path = substr(__FILE__,0,strpos(__FILE__,"wp-content"));
require($path . '/wp-load.php' );
nocache_headers();
wp();
require_once( ABSPATH . WPINC . '/template-loader.php' );
/** - END - Sets up the WordPress Environment. */


include(WP_PLUGIN_DIR.'/buddypress-group-wiki/bp-groupwiki.inc.php');

// Valid nonce (had to drop the URI part unless we add a function to work it out)
if ( !wp_verify_nonce($_REQUEST['_wpnonce'], 'wiki_edit_save_' ) ) {
	die('Security check');
}

global $bp, $wpdb;

$groupID = $_POST['group_id'];
$postID = $_POST['post_id'];
$blogID = groups_get_groupmeta( $groupID , 'wiki_blog_id' );

// Get the currently selected page
$selectedPage = get_blog_post( $blogID , $postID );
// Get post permalink for later redirection
$returnLink = bpgw_get_permalink( $groupID , $postID );
// Set the loop_post for the selected page
switch_to_blog($blogID);
start_in_the_loop($selectedPage);

$pagePrivacy = get_post_meta( $postID , 'wiki_view_access' , 1 );
$pageName = $selectedPage->post_title;
$oldAuthor = $selectedPage->post_author;

// Update the post if the user is allowed to
if ( user_can_edit_wiki_page( $postID , $groupID ) ) {
	
	// It appears that wordpress sanitises the data already.
	// Info: http://codex.wordpress.org/Function_Reference/wp_insert_post
	// (update post calls insert post)
	
	// Update the page with the new data
	$newPostContent = $_POST['wiki-post-edit'];
	$newPost = array();
		$newPost['ID'] = $postID;
		$newPost['post_content'] = $newPostContent;
		$newPost['post_author'] = $selectedPage->post_author;
	wp_update_post ( $newPost );
	// Fix the author problem - need to manually set the author for the old revision
	$revisions = wp_get_post_revisions($postID, array('numberposts' => 1, 'post_type' => 'revision',
												  'order' => 'DESC', 'orderby' => 'date'));
	if(!empty($revisions)) {
		// Get the post_id for the latest revision
		$latestRevisionID = $revisions[key($revisions)]->ID;
		// Update the revision with the author id of whoever was the previous current rev author
		$wpdb->query( $wpdb->prepare("UPDATE {$wpdb->base_prefix}%d_posts SET post_author = %d WHERE id =%s", $blogID , $oldAuthor , $latestRevisionID ) );
		// Now update the current rev (the parent post) with the current user's id
		$wpdb->query( $wpdb->prepare("UPDATE {$wpdb->base_prefix}%d_posts SET post_author = %d WHERE id =%s", $blogID , $bp->loggedin_user->id , $postID ) );
	}
	
	// Update the page restore history in the post metadata
	carry_over_restore_postmeta($postID);
	
	// Success feedback
	bp_core_add_message( __( 'Your update has been saved.', 'buddypress' ) );
	groups_update_groupmeta( $groupID, 'last_activity', time() );
	
	// Get all the editing post meta for this wikipage
	$postEditData = $wpdb->get_results( $wpdb->prepare("SELECT meta_id, meta_value FROM {$wpdb->base_prefix}%d_postmeta WHERE post_id = %d && meta_key = 'bpgw_edit_lock'", $blogID, $postID ) , ARRAY_A );
	
	if ( $postEditData ) { // Only do this if there's anything in there - there should always be at least the current user
		// Remove anyone else's editing postmeta that is older than 120seconds
		// Remove own editing postmeta
		// Set a warning flag on all remaining other editors postmeta so they'll get a one-time warning about this save
		foreach ( $postEditData as $postEdit) {		
			//explode the string with &
			//$editData['meta_value'][1]==userID, $editData['meta_value'][2]==timestamp, 
			//$editData['meta_value'][3]==Authorname, $editData['meta_value'][4]==warningflag (usually not set)
			$editData = explode( "&" , $postEdit['meta_value'] );
			if ( $editData[1] != $bp->loggedin_user->id ) {
				// This is someone else's editing meta.  If it's older than 15 seconds, delete it.  If it's newer than that, append save warning
				if ( time() - $editData[2] > 20 ) { // Old edit data.  Clean it from the db
					$wpdb->query( $wpdb->prepare( "	DELETE FROM {$wpdb->base_prefix}%d_postmeta
													WHERE meta_id = %d", 
													$blogID, $postEdit['meta_id'] ) );
				} else { // This is recent edit data, append the warning
					$meta_value = '&' . $editData[1] . '&' . time() . '&' . $editData[3] . '&1';
					$wpdb->query( $wpdb->prepare( "	UPDATE {$wpdb->base_prefix}%d_postmeta
													SET meta_value = '%s'
													WHERE meta_id = %d", 
													$blogID, $meta_value, $postEdit['meta_id'] ) );
				}
			} else { // This is the users own editing post meta - delete it
				$wpdb->query( $wpdb->prepare( "	DELETE FROM {$wpdb->base_prefix}%d_postmeta
												WHERE meta_id = %d", 
												$blogID, $postEdit['meta_id'] ) );
			}
		}
	}
	


	// Get last post revision so can do a diff to work out what to put in activity stream
	$previousRevisions = wp_get_post_revisions ( $selectedPage->ID );

	// Do this bit only if any revisions returned (should always be true due to the way wiki pages are created)
	if ( $previousRevisions ) {
		reset($previousRevisions);
		$revID = key($previousRevisions); 
		$lastRevision = $previousRevisions[$revID]; // Revision sorting defaults to newest > oldest so we just grab the first revision result
		$left_content = apply_filters( '_wp_post_revision_fields', $lastRevision->post_content  );  // Need these or things break
		$right_content  = apply_filters( '_wp_post_revision_fields',  get_post($postID)->post_content  );  // Need these or things break
		$left_content   = str_replace('&nbsp;','',$left_content);
		$right_content  = str_replace('&nbsp;','',$right_content);
		$revDiffContent = wp_text_diff( $left_content, $right_content );
		
		if (preg_match_all("/<td class=\'diff-addedline\'>(.*?)<\/td>/si", $revDiffContent, $lineText))
		{
			$addedContent = $lineText[0];
		}

		$thereIsContentAdded = 0;
		if ( isset($addedContent) ) {
			
			if ( count($addedContent) > 2 ) {
				$addedContent = array_slice($addedContent,-3,3); // Get the last three elements as this is probably more useful info
			}
			
			foreach ( $addedContent as $addedItem ) {
					$addItem = str_replace(array("\r\n", "\r", "\n"), '' , strip_tags( html_entity_decode($addedItem) ));
					if ( strlen( $addItem ) > 175 ) {
						$addItem = '...' . substr( $addItem , strlen($addItem)-175 , 175);
					} else {
						$addItem = substr( $addItem , 0 , 175);				
					}
					if  ( $addItem != '' ) {
						$addedText .= '<div>' . $addItem . '...</div>';
						$thereIsContentAdded++;
					}
			}
		} 

		if ( $thereIsContentAdded ) {
			
			$activityContent = $addedText;	
		
			$name = '<a href="' . $bp->loggedin_user->domain . '">' . $bp->loggedin_user->fullname . '</a>';
			
			$group = new BP_Groups_Group( $groupID , false , false );
			
			$activityUpdate = $name . ' edited the <a href="' . bp_get_group_permalink( $group ) . '">' . attribute_escape( $group->name ) . '</a> wiki page <a href="' . $returnLink . '">'.$pageName.'</a>: <span class="time-since">%s</span>';
			
			if ( $pagePrivacy == 'public' && $bp->groups->current_group->status != 'hidden' ) {
				// Record wiki creation in the activity stream, viewable by everyone
				wiki_groups_record_activity( array(
					'action' => $activityUpdate,
					'content' => $activityContent,
					'primary_link' => $returnLink,
					'component' => 'groups',
					'component_action' => 'new_wiki_edit',
					'type' => 'new_wiki_edit',
					'item_id' => $group->id,
					'secondary_item_id' => $bp->loggedin_user->id
				) );
			} else {
				// Record wiki creation in the activity stream, hide sitewide
				wiki_groups_record_activity( array(
					'content' => $activityUpdate,
					'primary_link' => $returnLink,
					'component' => 'groups',
					'component_action' => 'new_wiki_edit',
					'type' => 'new_wiki_edit',
					'item_id' => $group->id,
					'secondary_item_id' => $bp->loggedin_user->id,
					'hide_sitewide' => 1
				) );	
			}
		} 
	}

} else {

	// User isn't allowed to do the edit
	bp_core_add_message( __( 'You do not have the required access to do that.', 'buddypress' ), 'error' );
	
}
end_in_the_loop();
restore_current_blog();

// Send the user back to the page view screen
bp_core_redirect( $returnLink );

?>