<?php 
/*
 * Displays a summary of the current wiki pages the user can see within the group wiki. 
*/

	switch_to_wiki_blog( $bp->groups->current_group->id );

	query_posts('post_type=page&orderby=menu_order&order=ASC');
	remove_filter('the_content', 'table_of_contents', 9);

	// This will be set to true if any of the pages pass the user_can_view_wiki_page check
	$anyPagesVisible = false;

	// Build the summary rows
	$summaryRows = '';
	$newLine = "\n";
	
	$i = 0;
	while (have_posts()) : the_post();
		$currentPost = get_post(get_the_ID());
		if (user_can_view_wiki_page( get_the_ID() , $wikiGroupID ) ) {
			
			$pagename = '<a href="'.bpgw_get_permalink( $wikiGroupID , $currentPost->ID ).'">'.$currentPost->post_title.'</a>';
			$excerpt = wp_trim_excerpt($currentPost->content);
			$author = bp_core_get_userlink($currentPost->post_author);
			$date = to_wiki_date_format($currentPost->post_modified );
			
			$altStyle = '';
			if($i %2 == 0) $altStyle = ' class="alt"';
			$summaryRow = "<tr$altStyle>".$newLine;
			$summaryRow .= '<td class="col-pagename">'.$pagename.'</td>'.$newLine;
			$summaryRow .= '<td class="col-excerpt">'.$excerpt.'</td>'.$newLine;
			$summaryRow .= '<td class="col-author">'.$author.'</td>'.$newLine;
			$summaryRow .= '<td class="col-date">'.$date.'</td>'.$newLine;
			$summaryRow .= '</tr>'.$newLine;
			
			// Set $anyPagesVisible to true so that we know to return a non-false value
			$anyPagesVisible = true;
			$summaryRows .= $summaryRow;
			$i++;
		}
	endwhile;


	// If user can see any of these pages, return the html for the page links, else return false
	if ($anyPagesVisible) {
		// Build the table
		$output .= '<table id="wiki-summary">'.$newLine;
		$output .= '<thead>'.$newLine;
		$output .= '<tr>'.$newLine;
		$output .= '<th class="col-pagename">Page name</th>'.$newLine;
		$output .= '<th class="col-excerpt">Page excerpt</th>'.$newLine;
		$output .= '<th class="col-author">Last edited by</th>'.$newLine;
		$output .= '<th class="col-date">Last edited date</th>'.$newLine;
		$output .= '</tr>'.$newLine;
		$output .= '</thead>'.$newLine;
		$output .= '<tbody>'.$newLine;
		$output .= $summaryRows;
		$output .= '</tbody>'.$newLine;
		$output .= '</table>'.$newLine;
	} 
	else {
		$output .= '<p>You do not have permission to view any of the pages in this group wiki.</p>';
	}
		
		
?>
	<div id="blog-page" class="bp-widget">
		<div class="wiki-title">
			<h1>Group Wiki</h1>
			<?php if ($anyPagesVisible) { ?>
				<p class="wiki-meta">This table provides a summary of the current state of each wiki page you have access to.  To access a wiki page select a page 
				title link.</p>
			<?php } ?>
		</div>
		<?php echo $output; ?>
		
		
	<?php
	// This entire next section is for frontend wikipage creation.  
	
	if (groups_get_groupmeta( $bp->groups->current_group->id , 'wiki_member_page_create' ) == 1 && 
		groups_is_user_member( $bp->loggedin_user->id , $bp->groups->current_group->id ) ) {
	?>
		
		<script type="text/javascript">
				
		var frontendWikiPageCreateButtonAlreadyPressed = false;
		// Submits the frontend page creation stuff in wiki-index
		function frontendWikiPageCreate() {
		// 1. Set form stats = "clicked"
		// 2. Get the page title from form
		// 3. Clear the form data
		// 4. Submit the data via ajax
		// 5. Onsuccess: reload the page (future update:we'll make it update the page without a refresh)
			if ( !frontendWikiPageCreateButtonAlreadyPressed ) {
				var wikiPageTitle = jQuery("#wikiPageCreate").val();
				if ( wikiPageTitle != '' ) {
					frontendWikiPageCreateButtonAlreadyPressed = true;
					jQuery("#wikiPageCreate").val("loading...");
					jQuery.post("<?php echo site_url(); ?>?bpgw_group_id=<?php echo $wikiGroupID;?>&bpgw_newpage_title=" + wikiPageTitle + "&wiki_frontend_page_create=1", function(data){
						location.reload(true);
					});
				}
			}
		}
		</script>
		<br/>

		<form>
			<?php wp_nonce_field( 'wiki_frontend_newpage_'  );?> 
			<div id="wiki-frontend-page-create">
			    <div class="instruction">Create a new wiki page:</div>
				<input type="textarea" id="wikiPageCreate" name="wikiPageCreate" class="wiki-title-textarea" />
				<div class="generic-button group-button public" onclick="frontendWikiPageCreate();">
				<a href=""><span>Create Page</span></a>
				</div>
			</div>		
		</form>
		<?php
	} // END OF - the section is for frontend wikipage creation.  
	?>
	</div>
	
<?php restore_current_blog(); ?>