<?php
/*
 * This is the wiki group homepage screen.  This page redirects to appropriate 
 * script depending on user action.
*/

/************************************************************************************************************************
 *	I've added a few checks to this bit. 
 *	1) Get the page id/etc at this point so we can check for edit/view rights
 *	2) Add the check for invalid page/etc here as otherwise have to use redirects + causes header issues
 ************************************************************************************************************************/
 
// Turn debug on or off (1 or 0) 
$debug = 0;


global $bp;

switch_to_wiki_blog( $bp->groups->current_group->id );

// Set up filters to add in the various outputed elements
add_filter('the_content', 'table_of_contents', 9);


$wikiGroupID 		= $bp->groups->current_group->id;
$wikiBlogID 		= groups_get_groupmeta( $wikiGroupID , 'wiki_blog_id' );
$wikiPage 			= get_selected_page_from_url();

start_in_the_loop ( $wikiPage );

$wikiPageContent	= apply_filters( 'the_content' , $wikiPage->post_content );
$wikiPageRevisions	= wiki_page_revisions_summary( $wikiGroupID );
$wikiPageURI		= bpgw_get_permalink( $wikiGroupID , $wikiPage->ID );
$wikiActionPage		= $bp->action_variables[0]; // Page name
$wikiActionVar 	 	= $bp->action_variables[1]; // Page action ( edit/comment/revision )
$wikiCanEdit		= user_can_edit_wiki_page( $wikiPage->ID , $wikiGroupID );
$wikiCanView		= user_can_view_wiki_page( $wikiPage->ID , $wikiGroupID );

if ( $wikiPage ) {
	$wikiPageLinks		= wiki_page_links( $wikiGroupID , $wikiPage );
} else {
	$wikiPageLinks		= wiki_page_links( $wikiGroupID );
}
$wikiCanComment		= wiki_page_comments_enabled( $wikiPage );

// Only get comments if this is a page that the user can view
if ($wikiActionPage && $wikiCanView && $wikiCanComment) { 
	$wikiComments		= get_wiki_comments( $wikiPage );
	$wikiCommentTitleBar= get_comments_titlebar( $wikiPage->ID, count($wikiComments));
}

end_in_the_loop ();
restore_current_blog();

if ( $debug == 1 ) {
	echo '<pre>';
	echo '$wikiGroupID<br/>';
	var_dump ($wikiGroupID 	);
	echo '$wikiBlogID<br/>';
	var_dump ($wikiBlogID 	);
	echo '$wikiPage<br/>';
	var_dump ($wikiPage 	);
	echo '$wikiPageContent<br/>';
	var_dump ($wikiPageContent	);
	echo '$wikiPageURI<br/>';
	var_dump ($wikiPageURI 	);
	echo '$wikiPageActionPage<br/>';
	var_dump ($wikiActionPage );
	echo '$wikiActionVar<br/>';
	var_dump ($wikiActionVar  );
	echo '$wikiCanEdit<br/>';
	var_dump ($wikiCanEdit	);
	echo '$wikiCanView<br/>';
	var_dump ($wikiCanView	);
	echo '$wikiCanComment<br/>';
	var_dump ($wikiCanComment );
	echo '$wikiPageLinks<br/>';
	var_dump ($wikiPageLinks	);
	echo '</pre>';
}

// I've had to use this somewhat dodgy variable so that we can do the require statement AFTER echoing error/success messages
// Doh, in hindsight I realised a much cleaner way to do this.  I'll update this at some point.
$selectedPHPfile = ''; 

// Three steps to these checks:
// 1) If the wikiPageLinks var is set to false then no pages for the group are visible to this user, return an error and nothing else
// 2) If wikiPageLinks isn't false and there is no wikiActionPage set then this is the group wiki homepage.  Echo out the index page
// 3) If none of the above, must be on a wiki page and the user must be able to view at least something in the wiki.  
//	  Call the required action file after checking that the user can definitely do this

if ( !$wikiPageLinks ) { // User isn't allowed to view *any* of the wiki pages.  Send them packing

	bp_core_add_message( __( 'You do not have the required access to do that.', 'buddypress' ), 'error' );
	
} elseif ( !$wikiActionPage ) { // User is allowed to view some pages but doesn't have one selected.  Show them the index

	$selectedPHPfile = 'wiki-home.php';
	
} else { // User is trying to view a specific page or perform an action on one.  Check credentials and call appropriate file

	// Check the current page action (action_variables[1]) and load the needed action page after checking user access
	// If no action, assume this is a view request
	// NOTE - due to an update to buddypress I suspect these add messages aren't working anymore due to BP firing the event before any of the wiki page load.  We should probably implement our own method.
	switch ( $wikiActionVar ) {
		case 'edit':
			if ( $wikiCanEdit ) {
				$selectedPHPfile = 'wiki-edit.php';
			} else {
				$errorFeedback = 'You are not allowed to edit this page.';
			}
			break;
		case 'revision':
			if ( $wikiCanView ) {
				$selectedPHPfile = 'wiki-revision.php';
			} else {
				$errorFeedback = 'You are not allowed to view this page.';
			}
			break;
		default:
			if ( $wikiCanView ) {
				$selectedPHPfile = 'wiki-display.php';
			} else {
				$errorFeedback = 'You are not allowed to view this page.';
			}
	}
}

?>
<div id="groupwiki">
		
	<?php 
	if ( $wikiCanView && !empty($wikiPage)) {
		echo $wikiPageLinks;
	}
	
	if ( $wikiPageLinks == false && !$errorFeedback ) {
		?>
			<div id="message" class="info"><p>You do not currently have access to view any of the wiki pages of this group.</p></div>
		<?php
	} elseif ( $errorFeedback ) {
		?>
			<div id="message" class="error"><p><?php echo $errorFeedback;?></p></div>
		<?php
	}
	?>
	
	<?php if ( $selectedPHPfile ) require $selectedPHPfile; // Do whatever was selected from the switch statement above ?>
	
</div>