<?php
	define('REVISIONS_SYNTAX_ERROR', -1);
	define('REVISIONS_ALL', 0);
	define('REVISIONS_SINGLE', 1);
	define('REVISIONS_COMPARE', 2);
	
	// Get the group ID
	$group_id = $bp->groups->current_group->id;
	
	// Switch to the current blog for the group
	switch_to_wiki_blog( $group_id );
	
	// Get the currently selected page
	$currentPage = get_selected_page_from_url();
	
	$post_ID = $currentPage->ID;
	$userCanViewWiki = user_can_view_wiki_page($post_ID, $group_id);
	$userCanEditWiki = user_can_edit_wiki_page($post_ID, $group_id);

	// Grab the action variable from the URL
	$actionVar = $bp->action_variables[2];

	// Split it into an array of characters
	$chars = str_split($actionVar);	
	// Declare variables to help parse and store result
	$revisionsOp = null;
	$id1 = null;
	$id2 = null;
	$comparison = false;
	
	// if the action variable is empty then set op to show all revisions
	if(empty($chars[0]) || empty($chars)) {
		$revisionsOp = REVISIONS_ALL;
	}
	// else parse the action variable string char by char
	else {
		for($i = 0; $i < count($chars); $i++) {
			$char = $chars[$i];
			//echo("[i=".$i.", CHAR=".$char.", OP=".$revisionsOp.", flag=".($comparison==true)."] ");
			
			// Grab IDs in values are numeric
			if(is_numeric($char)) {
				if(!$comparison) {
					$id1 .= $char;
				} else {
					$id2 .= $char;
				}
			}
			// If character is not numeric, evaluate what are we dealing with
			if(!is_numeric($char)) {
				// Only the '-' char is allowed 
				// It must appear after the first ID has been collected
				// But must be followed by more characters (hopefully another ID)
				if($char == '-' && !empty($id1) && empty($comparison) && $i < count($chars)-1) {
					$comparison = true;
				} else {
					// Incorrect URL syntax, nullify all values and assume user is being naughty
					$revisionsOp = REVISIONS_SYNTAX_ERROR;
					$id1 = null;
					$id2 = null;
				}
			}
			// If a syntax error was found, break out of the loop early
			if($revisionsOp == REVISIONS_SYNTAX_ERROR) break;
		}
		// If two IDs were correctly parsed, must be a compare
		if($id1 != null && $id2 != null) {
				$revisionsOp = REVISIONS_COMPARE;
		}
		// If only one ID was found, must be viewing of a single revision
		elseif($id1 != null) {
			$revisionsOp = REVISIONS_SINGLE;
		}
	}
	//echo ("<p>Operation found was: " . $revisionsOp . " with ID1=".$id1." and ID2=".$id2."</p>");
	
	
	
	
	
	
	// Begin layout for operations
	?>
	<div id="blog-page" class="bp-widget">
		<div id='wiki-revisions' class='wiki-revisions'>
	<?php	
	
	// Select the output via a switch on the revision operation
	switch($revisionsOp) {
	
		case REVISIONS_SYNTAX_ERROR:
			echo '<div id="message" class="error"><p>The an incorrect URL appears to have been used to select revisions.  '.
				'Please select revisions via the interface rather than directly typing them in to your web browser.</p></div>';
			// Fall through...
			
		case REVISIONS_ALL:
			start_in_the_loop($currentPage);
			global $post;
			$titleBarArgs = array(
				'title_text' => "Full revision history for: ".$post->post_title,
				'restore_button' => false,
				'history_button' => false,
			);
			echo get_revisions_titlebar( $group_id , $titleBarArgs);
			echo "<p class='wiki-meta'>Select a revision date to view a previous version of the wiki page.  Alternatively select <em>two</em> revisions using the radio buttons followed by the 'Compare Revisions' button to view only the differences.</p>";
			// Grab all the revisions
			$revisions = get_revisions(-1);
			$comparisonTableHTML = format_revisions_as_comparison_table($group_id, $currentPage, $revisions);
			// Must restore current blog before we create the nonce else will cause error
			end_in_the_loop();
			restore_current_blog();
			
			?>
			<form id="revisionform" class="standard-form" method="post" action="<?php echo WIKI_S_REVISION_COMPARE ?>">
					<?php
					if ( function_exists('wp_nonce_field') )
						wp_nonce_field('wiki_revision_compare');
					?>
					<div class="form-submit">
					<input id="submit" class="submit-revision button" type="submit" value="Compare Revisions" tabindex="5" name="submit"/>
					</div>
					<?php echo $comparisonTableHTML; ?>
					<div class="form-submit">
					<input id="submit" class="submit-revision button" type="submit" value="Compare Revisions" tabindex="5" name="submit"/>
					<input type="hidden" id="group_id" name="group_id" value="<?php echo $wikiGroupID; ?>"/>
					<input type="hidden" id="post_id" name="post_id" value="<?php echo $wikiPage->ID; ?>"/>
					</div>
				</form>
			<?php
			break;
			
			
			
			
			
		case REVISIONS_SINGLE:
			// Set the loop_post for the selected page
			start_in_the_loop($currentPage);
			
			// This will store our selected revision
			$selectedRevision = null;
			$isRevisionCurrentVersion = false;
			
			if($id1 == $currentPage->ID) {
				$selectedRevision = $currentPage;
				$isRevisionCurrentVersion = true;
			}
			else {
				$selectedRevision = get_post($id1);
				// Check we have retrieved a valid revision post for this page
				if( $selectedRevision == null || // No post retrieved
					empty($selectedRevision) || // No post retrieved
					$selectedRevision->post_type != 'revision' || // Not a revision
					$selectedRevision->post_parent != $currentPage->ID ) // Not a revision belonging to the current page
				{
						echo "<p class='warning'>The selected revision ID doesn't appear to be valid for this page.</p>";
						break; // Exit early and only display the error message
				}
			}
			
			// If we get to this point, it should be safe to display the revision
			
			// Grab all the revisions for selected page
			$revisions = get_revisions(-1);
			
			// Calculate older and newer revisions
			$olderRevision = null;
			$newerRevision = null;
			$totalRevisions = count($revisions);
	
			if($isRevisionCurrentVersion) {
				$olderRevision = $revisions[0]; // The latest revision before the current one
			}
			else {
				$i=0;
				foreach ($revisions as $revision) {
					if($revision->ID == $selectedRevision->ID) {
						if($i<$totalRevisions-1) {
							$olderRevision = $revisions[$i+1];
						}
						if($i>0) {
							$newerRevision = $revisions[($i-1)];
						}
						break;
					}
					$i++;
				}

			}

			// Older revision link
			if($olderRevision != null) {
				$olderText = format_revision_as_title($olderRevision, true);
			} else {
				$olderText = "There are no older versions of the document.";
			}
			// Newer revision link
			if($newerRevision != null) {
				$newerText = format_revision_as_title($newerRevision, true);
			} 
			else if($isRevisionCurrentVersion) {
			}
			else {
				// The newer revision link must point to current page
				$newerText = format_revision_as_title($currentPage, true);
			}
			
			// Gather the data we need to output before we end the loop
			$selectedRevisionContent = apply_filters( 'the_content' , $selectedRevision->post_content );
			// Fix the broken image/file links caused by us being within the group directory.
			// This doesn't use apply_filters as I didn't want to hook this into the_content/etc, just use it in this one place
			$selectedRevisionContent = bpgw_revision_image_link_fix( $selectedRevisionContent );
			
			$fullHistoryLink = bpgw_get_permalink( $group_id , $currentPage->ID).'revision/';
			$revisionText = "Revision: ".format_revision_as_title($selectedRevision, false);
			$restoreURL = WIKI_S_REVISION_RESTORE."?group_id=$wikiGroupID&post_id=".$currentPage->ID."&revision_id=".$selectedRevision->ID;
			
			// Must restore current blog before we create the nonce else will cause error
			end_in_the_loop();
			restore_current_blog();
			
			// Append the nonce to the URL
			$restoreURL  = wp_nonce_url($restoreURL, 'wiki_revision_restore_');
			
			// Create all the arguments for the revisions title bar
			$titleBarArgs = array(
				'title_text' => $revisionText,
				'history_button' => true,
				'history_link' => $fullHistoryLink 
			);

			if($userCanViewWiki && $userCanEditWiki) {
				$titleBarArgs['restore_button'] = !$isRevisionCurrentVersion; // Only show if not viewing current page
				$titleBarArgs['restore_link'] = $restoreURL;
				$titleBarArgs['restore_link_attr'] = 'onclick="return confirm(\''.
										'Really restore this revision of the wiki to the current version?\n\n'.
										'To proceed with restoring this version, please select OK to confirm.\')"';
			}

			// Begin layout
			?>
			<div class='columns-two'>
				<div class='left'>
					<h5>View older revision</h5>
					<?php echo $olderText; ?>
				</div>
				<div class='right' style='text-align: right;'>
					<h5>View newer revision</h5>
					<?php echo $newerText; ?>
				</div>
			</div>
			<div class='clean'></div>
			<?php
			echo get_revisions_titlebar($group_id , $titleBarArgs);
			?>
				<h2><?php echo $selectedRevision->post_title; ?></h2>
				<div class="wiki-post">
					<?php 
					// If the post contains no content, display a message to inform user
					if($selectedRevision->post_content == "") {
						echo "<p>No content has been added to this page revision.</p>";
					}
					// Output the contents
					echo $selectedRevisionContent;
					?>
				</div>
			<?php
			break;
			
			
			
			
			
			
		case REVISIONS_COMPARE:
		
			// Check if IDs are identical
			if($id1 == $id2){
				echo '<div id="message" class="error"><p>You must select two different revisions and you appear to selected identical ones. '.
							 'Both versions are the same!</p></div>';
				break; // Exit early and only display the error message
			}

			// Set the loop_post for the selected page
			start_in_the_loop($currentPage);
						
			// This will store our selected revision
			$leftRevision = get_post($id1);
			$rightRevision = get_post($id2);

			if( !validateRevisionForCompare($currentPage, $leftRevision) ||
				!validateRevisionForCompare($currentPage, $rightRevision) )
			{
				echo "<p class='warning'>One or more of the selected revision IDs doesn't appear to be valid for this page.</p>";
				break; // Exit early and only display the error message
			}
				
			// If we get to this point, it should be safe to display the revision comparison
			
			// Create all the arguments for the revisions title bar
			$fullHistoryLink = bpgw_get_permalink( $group_id , $currentPage->ID).'revision/';
			$titleBarArgs = array(
				'title_text' => "Comparison of wiki page content",
				'history_button' => true,
				'history_link' => $fullHistoryLink 
			);
			
			// Get the content to compare
			$left_content   = $leftRevision->post_content;
			$right_content  = $rightRevision->post_content;
			
			$left_title		= format_revision_as_title($leftRevision, true);
			$right_title	= format_revision_as_title($rightRevision, true);
			
			// Tidy the HTML and then generate the diff
			$left_content   = str_replace('&nbsp;','',$left_content);
			$right_content  = str_replace('&nbsp;','',$right_content);
			$textDiffArgs 	= array('title_left' => $left_title, 'title_right' => $right_title);
			$output			= wp_text_diff( $left_content, $right_content, $textDiffArgs);
			//$output			= clean_revisions_diff_output($output);
			
			if(empty($output)) {
				$output  = "<p>Both these versions are <em>identical</em>. There are no differences to display.</p>";
				$output .= "<ul>\n<li>$left_title</li>\n<li>$right_title</li>\n</ul>";
			}
			
			// Display
			echo get_revisions_titlebar($group_id , $titleBarArgs); 
			echo $output;

			break;
		default:
			die("Logic error when calculating revision operation");
			break;
	}
	?>
		</div>
	</div>